import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import vma_ra_enabled_s390x
from insights.parsers.vma_ra_enabled_s390x import VmaRaEnabledS390x
from insights.tests import context_wrap

INPUT_VMA_1 = """
True
""".strip()

INPUT_VMA_2 = """
False
""".strip()


def test_vma_ra_enabled_s390x():
    sp1 = VmaRaEnabledS390x(context_wrap(INPUT_VMA_1))
    sp2 = VmaRaEnabledS390x(context_wrap(INPUT_VMA_2))
    assert sp1.ra_enabled is True
    assert sp2.ra_enabled is False


def test_vma_ra_enabled_s390x_exp():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipComponent) as sc1:
        VmaRaEnabledS390x(context_wrap(''))
    assert "Input content is empty" in str(sc1)


def test_vma_ra_enabled_s390x__documentation():
    """
    Here we test the examples in the documentation automatically using doctest.
    We set up an environment which is similar to what a rule writer might see -
    a '/sys/kernel/mm/swap/vma_ra_enabled' output that has been passed in as a
    parameter to the rule declaration.
    """
    env = {'vma': VmaRaEnabledS390x(context_wrap(INPUT_VMA_1))}

    failed, total = doctest.testmod(vma_ra_enabled_s390x, globs=env)
    assert failed == 0



"""
IpsecConf parser - file ``/etc/ipsec.conf``
===========================================

IpsecConf parser the file /etc/ipsec.conf about
the configuration and control information
for the Libreswan IPsec subsystem.
"""
from collections import defaultdict

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.ipsec_conf)
class IpsecConf(CommandParser, dict):
    """
    Class for parsing the file ``/etc/ipsec.conf`` about the configuration
    and control information for the Libreswan IPsec subsystem

    Raises:
        SkipComponent: When content is empty or cannot be parsed.

    Sample output of this command is::

        # /etc/ipsec.conf - Libreswan IPsec configuration file
        #
        # see 'man ipsec.conf' and 'man pluto' for more information
        #
        # For example configurations and documentation, see https://libreswan.org/wiki/

        config setup
                # plutodebug="control parsing"
                # plutodebug="all crypt"
                plutodebug=none
                # It seems that T-Mobile in the US and Rogers/Fido in Canada are
                # using 25/8 as "private" address space on their wireless networks.
                # This range has never been announced via BGP (at least up to 2015)
                virtual_private=%v4:10.0.0.0/8,%v4:192.168.0.0/16,%v4:172.16.0.0/12,%v4:25.0.0.0/8,%v4:100.64.0.0/10,%v6:fd00::/8,%v6:fe80::/10

        # if it exists, include system wide crypto-policy defaults
        include /etc/crypto-policies/back-ends/libreswan.config

        # It is best to add your IPsec connections as separate files in /etc/ipsec.d/
        include /etc/ipsec.d/*.conf

    Examples:
        >>> ipsec_conf['config']['setup']['plutodebug'] == 'none'
        True
        >>> ipsec_conf['include']
        ['/etc/crypto-policies/back-ends/libreswan.config', '/etc/ipsec.d/*.conf']
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent('No content.')

        ipsec_type, ipsec_name = "", ""
        ipsec_sections = {}

        try:
            for line in get_active_lines(content):
                if line.startswith('include '):
                    include, path = [field.strip() for field in line.split()]
                    array = self.get('include', [])
                    array.append(path)
                    self['include'] = array
                    continue

                if line.startswith(('conn ', 'config ')):
                    ipsec_type, ipsec_name = [field.strip() for field in line.split()]
                    ipsec_sections = self.get(ipsec_type, defaultdict(dict))
                    continue

                if '=' not in line or ipsec_type == "" or ipsec_name == "":
                    # skip the options that don't within a section
                    continue

                key, value = [field.strip() for field in line.split('=')]
                ipsec_sections[ipsec_name][key] = value
                self[ipsec_type] = ipsec_sections

        except ValueError:
            raise SkipComponent('Syntax error')



"""
Custom datasource to get the linux users count who map to a selinux user.
"""

import json

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.parsers import parse_fixed_table
from insights.specs import Specs


class LocalSpecs(Specs):
    """Local specs used only by users_count_map_selinux_user datasource"""

    selinux_user_mapping = simple_command("/sbin/semanage login -l")


@datasource(LocalSpecs.selinux_user_mapping, HostContext)
def users_count_map_selinux_user(broker):
    """
    Get linux users count who map to the selinux user. It returns a dict with
    the selinux user type as the key, and the linux users count as the value.

    Raises:
        SkipComponent: Raised when there are no users map the filtered selinux user.
    """
    user_mapping_lines = broker[LocalSpecs.selinux_user_mapping].content
    data = parse_fixed_table(
        user_mapping_lines,
        heading_ignore=['Login Name'],
        header_substitute=[
            ('Login Name', 'login_name'),
            ('SELinux User', 'selinux_user'),
            ('MLS/MCS Range', 'mls_mcs_range'),
            ('Service', 'service'),
        ],
    )
    filters = sorted((get_filters(Specs.selinux_users)))
    users_info = {}
    for filter in filters:
        users_info[filter] = len([obj for obj in data if obj['selinux_user'] == filter])
    if users_info:
        return DatasourceProvider(
            json.dumps(users_info),
            relative_path='insights_datasources/linux_users_count_map_selinux_user',
            ds=Specs.users_count_map_selinux_user,
            ctx=broker.get(HostContext),
            cleaner=broker.get("cleaner"),
        )
    raise SkipComponent



from insights.client.connection import InsightsConnection
from mock.mock import Mock, MagicMock, patch


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.constants.cached_branch_info", "/tmp/insights-test-cached-branchinfo")
def test_request(get_proxies, init_session):
    """
    The request to get branch info is issued.
    """
    config = Mock(base_url="www.example.com", branch_info_url="https://www.example.com/branch_info")

    connection = InsightsConnection(config)
    connection.get = MagicMock()
    connection.get_branch_info()

    connection.get.assert_called_once_with(config.branch_info_url)



import requests
import json
from insights.client.connection import InsightsConnection
from mock.mock import MagicMock, Mock, patch


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_ok_reg(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Request completed OK, registered
        Returns True
    '''
    config = Mock(legacy_upload=True, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps({'unregistered_at': None})
    res.status_code = 200

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check()


@patch("insights.client.connection.generate_machine_id", return_value="12345678-abcd-efgh-ijkl-mnopqstvwxyz")
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.create_system")
def test_register_ok_reg_group(mock_create_system, _get_proxies, _init_session, _generate_machine_id):
    """
    Test the case, when we try to add system to existing group
    """

    # Mock creating connection
    config = Mock(
        legacy_upload=True,
        base_url="example.com",
        group="my_testing_group_name",
    )
    conn = InsightsConnection(config)

    # Mock creating system on the server
    mock_create_system.return_value = Mock(status_code=200)

    # Mock getting list of groups
    get_groups_res = requests.Response()
    group_list = {
        "count": 1,
        "page": 1,
        "per_page": 1,
        "total": 1,
        "results": [
            {
                "created": "2024-06-20T15:39:54.888Z",
                "id": "87654321-dcba-efgh-ijkl-123456789abc",
                "name": "my_testing_group_name",
                "org_id": "000102",
                "updated": "2024-06-20T15:39:54.888Z",
                "host_count": 0
            },
        ]
    }
    get_groups_res.encoding = "utf-8"
    get_groups_res._content = json.dumps(group_list).encode("utf-8")
    get_groups_res.status_code = 200
    conn.get = MagicMock(return_value=get_groups_res)

    # Mock putting system to the group
    updated_group = {
        "created": "2024-06-21T10:32:27.135Z",
        "id": "bA6deCFc19564430AB814bf8F70E8cEf",
        "name": "my_testing_group_name",
        "org_id": "000102",
        "updated": "2024-06-21T10:32:27.135Z",
        "host_count": 1
    }
    post_system_to_group_res = requests.Response()
    post_system_to_group_res.encoding = "utf-8"
    post_system_to_group_res._content = json.dumps(updated_group).encode("utf-8")
    post_system_to_group_res.status_code = 201
    conn.post = MagicMock(return_value=post_system_to_group_res)

    # Test the ^%&$#@* code
    _, _, group_name, _ = conn.register()
    assert group_name == "my_testing_group_name"


@patch("insights.client.connection.generate_machine_id", return_value="12345678-abcd-efgh-ijkl-mnopqstvwxyz")
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.create_system")
def test_register_ok_reg_empty_group(mock_create_system, _get_proxies, _init_session, _generate_machine_id):
    """
    Test the case, when we try to add system to group that does not exist,
    but server response with empty list and not 404 code
    """

    # Mock creating connection
    config = Mock(
        legacy_upload=True,
        base_url="example.com",
        group="my_testing_group_name",
    )
    conn = InsightsConnection(config)

    # Mock creating system on the server
    mock_create_system.return_value = Mock(status_code=200)

    # Mock getting the list of groups
    get_groups_res = requests.Response()
    group_list = {
        "count": 0,
        "page": 0,
        "per_page": 0,
        "total": 0,
        "results": []
    }
    get_groups_res.encoding = "utf-8"
    get_groups_res._content = json.dumps(group_list).encode("utf-8")
    get_groups_res.status_code = 200
    conn.get = MagicMock(return_value=get_groups_res)

    # Mock response of creating new group with new system
    new_group = {
        "created": "2024-06-21T12:25:21.009Z",
        "id": "bA6deCFc19564430AB814bf8F70E8cEf",
        "name": "my_testing_group_name",
        "org_id": "000102",
        "updated": "2024-06-21T12:25:21.009Z",
        "host_count": 1
    }
    post_system_to_group_res = requests.Response()
    post_system_to_group_res.encoding = "utf-8"
    post_system_to_group_res._content = json.dumps(new_group).encode("utf-8")
    post_system_to_group_res.status_code = 201
    conn.post = MagicMock(return_value=post_system_to_group_res)

    # Test the code
    _, _, group_name, _ = conn.register()
    assert group_name == "my_testing_group_name"


@patch("insights.client.connection.generate_machine_id", return_value="12345678-abcd-efgh-ijkl-mnopqstvwxyz")
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.create_system")
def test_register_ok_reg_non_existing_group(mock_create_system, _get_proxies, _init_session, _generate_machine_id):
    """
    Test the case, when we try to add the system to a group that has not been already created,
    and it is necessary to create the group during registration process.
    """

    # Mock creating connection
    config = Mock(
        legacy_upload=True,
        base_url="example.com",
        group="my_testing_group_name",
    )
    conn = InsightsConnection(config)

    # Mock creating system on the server
    mock_create_system.return_value = Mock(status_code=200)

    # Mock getting list of groups
    get_groups_res = requests.Response()
    get_groups_res.encoding = "utf-8"
    get_groups_res.status_code = 404
    conn.get = MagicMock(return_value=get_groups_res)

    # Mock response of creating new group with new system
    new_group = {
        "created": "2024-06-21T12:25:21.009Z",
        "id": "bA6deCFc19564430AB814bf8F70E8cEf",
        "name": "my_testing_group_name",
        "org_id": "000102",
        "updated": "2024-06-21T12:25:21.009Z",
        "host_count": 1
    }
    post_system_to_group_res = requests.Response()
    post_system_to_group_res.encoding = "utf-8"
    post_system_to_group_res._content = json.dumps(new_group).encode("utf-8")
    post_system_to_group_res.status_code = 201
    conn.post = MagicMock(return_value=post_system_to_group_res)

    # Test the code
    _, _, group_name, _ = conn.register()
    assert group_name == "my_testing_group_name"


@patch("insights.client.connection.generate_machine_id", return_value="12345678-abcd-efgh-ijkl-mnopqstvwxyz")
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.create_system")
def test_register_failed_reg_non_existing_group(mock_create_system, _get_proxies, _init_session, _generate_machine_id):
    """
    Test the case, when we try to add the system to a group that has not been already created,
    but it is not possible to create the group during registration process.
    """

    # Mock creating connection
    config = Mock(
        legacy_upload=True,
        base_url="example.com",
        group="my_testing_group_name",
    )
    conn = InsightsConnection(config)

    # Mock creating system on the server
    mock_create_system.return_value = Mock(status_code=200)

    # Mock getting list of groups
    get_groups_res = requests.Response()
    get_groups_res.encoding = "utf-8"
    get_groups_res.status_code = 404
    conn.get = MagicMock(return_value=get_groups_res)

    # Mock response of not creating new group due to lack of permissions
    post_system_to_group_res = requests.Response()
    post_system_to_group_res.encoding = "utf-8"
    post_system_to_group_res.status_code = 403
    conn.post = MagicMock(return_value=post_system_to_group_res)

    # Test the code
    _, _, group_name, _ = conn.register()
    assert group_name == "my_testing_group_name"


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_ok_reg_then_unreg(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Request completed OK, was once registered but has been unregistered
        Returns the date it was unregistered
    '''
    config = Mock(legacy_upload=True, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps({'unregistered_at': '2019-04-10'})
    res.status_code = 200

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() == '2019-04-10'


@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_ok_unreg(get_proxies, _init_session, _machine_id_exists):
    '''
    Request completed OK, has never been registered
        Returns None
    '''
    config = Mock(legacy_upload=True, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = json.dumps({})
    res.status_code = 404

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is None


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_bad_res(get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Can't parse response
        Returns False
    '''
    config = Mock(legacy_upload=True, base_url='example.com')
    conn = InsightsConnection(config)

    res = requests.Response()
    res._content = 'zSDFasfghsRGH'
    res.status_code = 500

    conn.get = MagicMock(return_value=res)
    assert conn.api_registration_check() is False


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.test_connection")
def test_registration_check_conn_error(test_connection, get_proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Can't connect, run connection test
        Returns False
    '''
    config = Mock(legacy_upload=True, base_url='example.com')
    conn = InsightsConnection(config)

    conn.get = MagicMock()
    conn.get.side_effect = requests.ConnectionError()
    assert conn.api_registration_check() is False
    test_connection.assert_called_once()



"""
LS Parsers - command ``ls``
===========================

Parsers provided in this module includes:

LSla - command ``ls -la <dirs>``
--------------------------------

LSlaFiltered - command ``ls -la <dirs> | grep -F <keywords>``
-------------------------------------------------------------

LSlan - command ``ls -lan <dirs>``
----------------------------------

LSlanFiltered - command ``ls -lan <dirs> | grep -F <keywords>``
---------------------------------------------------------------

LSlanL - command ``ls -lanL <dirs>``
------------------------------------

LSlanR - command ``ls -lanR <dirs>``
------------------------------------

LSlanRL - command ``ls -lanRL <dirs>``
--------------------------------------

LSlaRZ - command ``ls -lanRZ <dirs>``
-------------------------------------

LSlaZ - command ``ls -lanZ <dirs>``
-----------------------------------

LSldH - command ``ls -ldH <items>``
-----------------------------------

LSldZ - command ``ls -ldZ <items>``
-----------------------------------

LSlHFiles - spec ``ls_files`` -  command ``ls -lH  <files>``
------------------------------------------------------------
"""

from insights.core import ls_parser, CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.filters import add_filter
from insights.core.ls_parser import FilePermissions
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import deprecated

# Required basic filters for `LS` specs that the content needs to be filtered
add_filter(Specs.ls_la_filtered, ['total '])
add_filter(Specs.ls_lan_filtered, ['total '])


def compose_raw_entry(target):
    if target:
        raw_line = ' '.join([target['type'] + target['perms']])
        if 'links' in target:
            raw_line += ' ' + str(target['links'])
        raw_line += ' ' + ' '.join([target['owner'], target['group']])
        if 'se_user' in target:
            raw_line += ' ' + ':'.join(
                [target['se_user'], target['se_role'], target['se_type'], target['se_mls']]
            )
        if 'size' in target:
            raw_line += ' ' + str(target['size'])
        if 'major' in target:
            raw_line += ' ' + ', '.join([str(target['major']), str(target['minor'])])
        if 'date' in target:
            raw_line += ' ' + target['date']
        raw_line += ' ' + target['name']
        if target['type'] == 'l' and 'link' in target:
            raw_line += ' -> ' + target['link']
        return raw_line


class FileListing(CommandParser, dict):
    """
    Reads a series of concatenated directory listings and turns them into
    a dictionary of entities by name.  Stores all the information for
    each directory entry for every entry that can be parsed, containing::

        - type (one of [bcdlps-])
        - permission string including ACL character
        - number of links
        - owner and group (as given in the listing)
        - size, or major and minor number for block and character devices
        - date (in the format given in the listing)
        - name
        - name of linked file, if a symlink

    In addition, the raw line is always stored, even if the line doesn't look
    like a directory entry.

    Also provides a number of other conveniences, such as::

        - lists of regular and special files and subdirectory names for each
          directory, in the order found in the listing
        - total blocks allocated to all the entities in this directory

    Parses the SELinux information if present in the listing.
    SELinux directory listings contain::

        - the type of file
        - the permissions block
        - the owner and group as given in the directory listing
        - the SELinux user, role, type and MLS
        - the name, and link destination if it's a symlink

    .. note::
        The :class:`FileListing` parses the content collected by
        diffirent ``ls_*`` specs. The ``ls_*`` specs collect the corresponding
        ``ls`` command output according to the filters defined by the relevant
        ``ls_*_dirs`` specs.  For the ``ls_*_dirs`` specs, only absolute
        directory path is acceptable, path to file or relative path is not
        acceptable.  For details, see the following example.

    Sample output::

        /boot:
        total 187380
        dr-xr-xr-x.  3 0 0   4096 Mar  4 16:19 .
        dr-xr-xr-x. 19 0 0   4096 Jul 14 09:10 ..
        -rw-r--r--.  1 0 0 123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64

        /etc/sysconfig:
        total 96
        drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
        drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
        drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
        drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
        -rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
        -rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
        lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

    Examples:
        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.ls_lan_dirs, ['/boot', '/etc/sysconfig'])
        >>> type(ls_lan)
        <class 'insights.parsers.ls.LSlan'>
        >>> "/etc" in ls_lan
        False
        >>> "/etc/sysconfig" in ls_lan
        True
        >>> len(ls_lan.files_of('/etc/sysconfig'))
        3
        >>> ls_lan.files_of("/etc/sysconfig")
        ['ebtables-config', 'firewalld', 'grub']
        >>> ls_lan.dirs_of("/etc/sysconfig")
        ['.', '..', 'cbq', 'console']
        >>> ls_lan.specials_of("/etc/sysconfig")
        []
        >>> ls_lan.total_of("/etc/sysconfig")
        96
        >>> ls_lan.dir_entry('/etc/sysconfig', 'grub') == {'group': '0', 'name': 'grub', 'links': 1, 'perms': 'rwxrwxrwx.', 'owner': '0', 'link': '/etc/default/grub', 'date': 'Jul  6 23:32', 'type': 'l', 'dir': '/etc/sysconfig', 'size': 17}
        True
        >>> sorted(ls_lan.listing_of("/etc/sysconfig").keys()) == sorted(['console', 'grub', '..', 'firewalld', '.', 'cbq', 'ebtables-config'])
        True
        >>> sorted(ls_lan.listing_of("/etc/sysconfig")['console'].keys()) == sorted(['group', 'name', 'links', 'perms', 'owner', 'date', 'type', 'dir', 'size'])
        True
        >>> ls_lan.listing_of("/etc/sysconfig")['console']['type']
        'd'
        >>> ls_lan.listing_of("/etc/sysconfig")['console']['perms']
        'rwxr-xr-x.'
        >>> ls_lan.dir_contains("/etc/sysconfig", "console")
        True
        >>> ls_lan.dir_entry("/etc/sysconfig", "console") == {'group': '0', 'name': 'console', 'links': 2, 'perms': 'rwxr-xr-x.', 'owner': '0', 'date': 'Sep 16  2015', 'type': 'd', 'dir': '/etc/sysconfig', 'size': 6}
        True
        >>> ls_lan.dir_entry("/etc/sysconfig", "grub")['type']
        'l'
        >>> ls_lan.dir_entry("/etc/sysconfig", "grub")['link']
        '/etc/default/grub'
        >>> "/boot" in ls_lan
        True
        >>> ls_lan.files_of('/boot')
        ['config-3.10.0-229.14.1.el7.x86_64']
        >>> fp = ls_lan.permissions_of('/boot', 'config-3.10.0-229.14.1.el7.x86_64')
        >>> fp.owner
        '0'
        >>> fp.group
        '0'
        >>> fp.perms_owner
        'rw-'
    """

    __root_path = None
    """
    The root path of the dir when there is only one list target.  It only works
    for the following specs/parsers that are compatible for sos-archives.
    - :class:`insights.parsers.ls_boot.LsBoot`
    - :class:`insights.parsers.ls_dev.LsDev`
    - :class:`insights.parsers.ls_sys_firmware.LsSysFirmware`

    None by default, for the new ```ls_*``` datasource specs
    """

    def parse_content(self, content):
        """
        Called automatically to process the directory listing(s) contained in
        the content.
        """
        self.update(ls_parser.parse(content, self.__root_path))

    def files_of(self, directory):
        """
        The list of non-special files (i.e. not block or character files)
        in the given directory.
        """
        if directory in self:
            return self[directory]['files']
        return []

    def dirs_of(self, directory):
        """
        The list of subdirectories in the given directory.
        """
        if directory in self:
            return self[directory]['dirs']
        return []

    def specials_of(self, directory):
        """
        The list of block and character special files in the given directory.
        """
        if directory in self:
            return self[directory]['specials']
        return []

    def total_of(self, directory):
        """
        The total blocks of storage consumed by entries in this directory.
        """
        if directory in self:
            return self[directory]['total']
        return 0

    def listing_of(self, directory):
        """
        The listing of this directory, in a dictionary by entry name.
        Entries that can be parsed then have fields as described in the class
        description above.

        .. note::
            The 'raw_entry' key is removed from the return value.  Use the
            `raw_entry_of` method instead.
        """
        if directory in self:
            return self[directory]['entries']
        return {}

    def dir_contains(self, directory, name):
        """
        Does this directory contain this entry name?
        """
        if directory in self:
            return name in self[directory]['entries']
        return False

    def dir_entry(self, directory, name):
        """
        The parsed data for the given entry name in the given directory.
        """
        if directory in self:
            return self[directory]['entries'][name]
        return {}

    def path_entry(self, path):
        """
        The parsed data given a path, which is separated into its directory
        and entry name.
        """
        if path[0] != '/':
            return None
        path_parts = path.split('/')
        # Note that here the first element will be '' because it's before the
        # first separator.  That's OK, the join puts it back together.
        directory = '/'.join(path_parts[:-1])
        if directory not in self:
            return None
        name = path_parts[-1]
        if name not in self[directory]['entries']:
            return None
        return self[directory]['entries'][name]

    def raw_entry_of(self, directory, target):
        """
        Returns the raw line entry of the ``directory``/``target`` listed in
        'ls' command output.

        Parameters:
            directory(string): Full path without trailing slash where to
                search.
            target (string): Name of the directory or file to get
                FilePermissions for.

        Returns:
            str: The re-constructed rough line if found or None if not found.

        .. note::
            As it's re-constructed according to the serialized items, it's not
            identical with the original line.
        """
        if directory in self:
            de = self[directory]['entries']
            return compose_raw_entry(de.get(target))

    def permissions_of(self, directory, target):
        """
        Returns a FilePermissions object, if found.

        Parameters:
            directory(string): Full path without trailing slash where to
                search.
            target (string): Name of the directory or file to get
                FilePermissions for.

        Returns:
            FilePermissions: If found or None if not found.
        """
        raw_line = self.raw_entry_of(directory, target)
        if raw_line:
            return FilePermissions(raw_line)


@parser(Specs.ls_la)
class LSla(FileListing):
    """
    Parses output of ``ls -la <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_la_filtered)
class LSlaFiltered(FileListing):
    """
    Parses output of ``ls -la <dirs> | grep -F <keywords>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_lan)
class LSlan(FileListing):
    """
    Parses output of ``ls -lan <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_lan_filtered)
class LSlanFiltered(FileListing):
    """
    Parses output of ``ls -lan <dirs> | grep -F <keywords>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_lanL)
class LSlanL(FileListing):
    """
    Parses output of ``ls -lanR <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_lanR)
class LSlanR(FileListing):
    """
    Parses output of ``ls -lanR <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_lanRL)
class LSlanRL(FileListing):
    """
    Parses output of ``ls -lanRL <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_laRZ)
class LSlaRZ(FileListing):
    """
    Parses output of ``ls -laRZ <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_laZ)
class LSlaZ(FileListing):
    """
    Parses output of ``ls -laZ <dirs>`` command.
    See :py:class:`FileListing` for more information.
    """

    pass


@parser(Specs.ls_files)
class LSlHFiles(CommandParser, dict):
    """
    Parses file information of ``ls -lH`` command.

    .. warning::
        This class is deprecated and will be removed from 3.7.0.
        Please use the :class:`LSldH` instead.

    .. note::

        To parse a specific file, its full path should be added to the
        `ls_lH_files` spec via `add_filter`.
        Only paths point to files are acceptable.

    """
    def __init__(self, *args, **kwargs):
        deprecated(
            LSlHFiles, "Please use the :class:`LSldH` instead.", "3.7.0"
        )
        super(LSlHFiles, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.error_lines = []
        for line in content:
            if "ls: cannot access" in line and "No such file or directory" in line:
                self.error_lines.append(line)
                continue
            try:
                line = line.strip()
                self[line.rsplit(None, 1)[-1]] = FilePermissions(line)
            except Exception:
                pass
        if not self:
            raise SkipComponent


class FileListingNoHeader(CommandParser, dict):
    """
    Parses a flat, long-listing with SELinux context information
    where each entry represents an absolute path and no directory
    headers are present. Stores all information for each successfully
    parsed entry, containing::

        - type:     File type indicator ('d', '-', 'l', 'b', 'c')
        - perms:    Permission bits, e.g. 'rw-r--r--.'
        - links:    Number of hard links
        - owner:    Owner user name
        - group:    Owner group name
        - size:     File size in bytes (if applicable)
        - major:    Device major number (for block/char devices)
        - minor:    Device minor number (for block/char devices)
        - se_user:  SELinux user (if applicable)
        - se_role:  SELinux role (if applicable)
        - se_type:  SELinux type (if applicable)
        - se_mls:   SELinux level (if applicable)
        - name:     Full absolute path to the file
        - date:     Modification date/time string
        - link:     Target path (if the entry is a symlink)
        - dir:      Parent directory (if applicable)

    Attributes::

        entries (dict): A mapping of all parsed entries, keyed by their absolute paths.
        files (list): The list of all non-special files (i.e. not block or character devices).
        dirs (list): The list of all directories parsed.
        specials (list): The list of all special files (block and character devices).
        error_lines (list): The list of all special files (block and character devices).

    Sample output::

        ls: cannot access '/dev/socketfile': No such file or directory
        dr-xr-xr-x. 5 root root     4096 May 30 06:57 /boot
        drwx------. 4 root root       83 Nov  6  2024 /boot/grub2
        -rw-------. 1 root root     6658 Dec 20  2023 /boot/grub2/grub.cfg
        crw--w----. 1 root tty  136,   0 Nov 17 01:28 /dev/stderr
        brw-rw----. 1 root disk 252,   0 Nov 16 03:19 /dev/vda
        crw-------. 1 root root  10, 137 Nov  6  2024 /dev/vhci

    Examples::

        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.ls_ldH_items, ['/boot', '/boot/grub2/grub.cfg', '/dev/stderr', '/dev/vda', '/dev/vhci'])
        >>> type(ls_files_no_header)
        <class 'insights.parsers.ls.FileListingNoHeader'>
        >>> len(ls_files_no_header.entries.keys())
        6
        >>> "/boot/grub2/grub.cfg" in ls_files_no_header.files
        True
        >>> ls_files_no_header.dirs
        ['/boot', '/boot/grub2']
        >>> "/dev/vda" in ls_files_no_header.specials
        True
        >>> ls_files_no_header.entries['/boot']['perms']
        'r-xr-xr-x.'
    """

    def parse_content(self, content):
        """
        Parses a file/directory listing with SELinux information from the
        provided content and updates the dictionary with all parsed entries.
        """
        result = ls_parser.parse(content, None)
        self.update(result.get(None))
        self.error_lines = result.pop('error_lines') if 'error_lines' in result else []

    entries = property(lambda self: self['entries'])
    files = property(lambda self: self['files'])
    dirs = property(lambda self: self['dirs'])
    specials = property(lambda self: self['specials'])

    def raw_entry_of(self, target):
        """
        Returns the raw line entry of the ``target`` listed in
        'ls' command output.

        Parameters:
            target (string): Name of the directory or file to get
                information for.

        Returns:
            str: The re-constructed rough line if found or None if not found.

        .. note::
            As it's re-constructed according to the serialized items, it's not
            identical with the original line.
        """
        return compose_raw_entry(self['entries'].get(target))

    def permissions_of(self, target):
        """
        Returns a FilePermissions object, if found.

        Parameters:
            directory(string): Full path without trailing slash where to
                search.
            target (string): Name of the directory or file to get
                FilePermissions for.

        Returns:
            FilePermissions: If found or None if not found.
        """
        raw_line = self.raw_entry_of(target)
        if raw_line:
            return FilePermissions(raw_line)


@parser(Specs.ls_ldH)
class LSldH(FileListingNoHeader):
    """
    Parses output of ``ls -ldH`` command.
    See :py:class:`FileListingNoHeader` for more information.

    .. note::

        To parse a specific file, its full path should be added to the
        `ls_ldH_items` spec via `add_filter`.

    """
    pass


@parser(Specs.ls_ldZ)
class LSldZ(FileListingNoHeader):
    """
    Parses output of ``ls -ldZ`` command.
    See :py:class:`FileListingNoHeadSelinux` for more information.

    .. note::

        To parse a specific file, its full path should be added to the
        `ls_ldZ_items` spec via `add_filter`.

    """
    pass



"""
YumUpdates - parser for the `yum_updates` datasource
====================================================
Provides a list of available package updates, along with related advisories. This information
is collected using DNF/YUM python interface.
"""

from insights import JSONParser, parser
from insights.specs import Specs


@parser(Specs.yum_updates)
class YumUpdates(JSONParser):
    """
    Expected output of the command is::

        {
          "releasever": "8",
          "basearch": "x86_64",
          "update_list": {
            "NetworkManager-1:1.22.8-4.el8.x86_64": {
              "available_updates": [
                {
                  "package": "NetworkManager-1:1.22.8-5.el8_2.x86_64",
                  "repository": "rhel-8-for-x86_64-baseos-rpms",
                  "basearch": "x86_64",
                  "releasever": "8",
                  "erratum": "RHSA-2020:3011"
                }
              ]
            }
          },
          "metadata_time": "2021-01-01T09:39:45Z"
        }

    Examples:
        >>> len(updates.updates)
        1
        >>> updates.updates['NetworkManager-1:1.22.8-4.el8.x86_64']['available_updates'][0] == { \
        'basearch': 'x86_64', \
        'erratum': 'RHSA-2020:3011', \
        'package': 'NetworkManager-1:1.22.8-5.el8_2.x86_64', \
        'releasever': '8', \
        'repository': 'rhel-8-for-x86_64-baseos-rpms'}
        True
    """

    @property
    def updates(self):
        """
        Returns:
            dict: Dict(package name -> list of available updates)
        """
        return self.data['update_list']



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import sys_block
from insights.tests import context_wrap

STABLE_WRITES = """
1
""".strip()

CONTENT_INVALID = """
invalid
""".strip()

CONTENT_EMPTY = """
""".strip()

MAX_SEGMENT_SIZE = """
4294967295
""".strip()

FILE_PATH_SDA = "/sys/block/sda/queue/scheduler"
MAX_SEGMENT_SIZE_PATH_SDA = "/sys/block/sda/queue/max_segment_size"
SPECIAL_DEVICE_PATH = "/sys/block/sd@!$/queue/max_segment_size"


def test_stable_writes():
    res = sys_block.StableWrites(context_wrap(STABLE_WRITES, FILE_PATH_SDA))
    assert res.stable_writes == 1
    assert res.device == 'sda'


def test_invalid_stable_writes():
    with pytest.raises(ParseException) as e:
        sys_block.StableWrites(context_wrap(CONTENT_INVALID, FILE_PATH_SDA))
    assert "Error: " in str(e)

    with pytest.raises(ParseException) as e:
        sys_block.StableWrites(context_wrap(CONTENT_EMPTY))
    assert "Error: " in str(e)


def test_max_segment_size():
    res = sys_block.MaxSegmentSize(context_wrap(MAX_SEGMENT_SIZE, MAX_SEGMENT_SIZE_PATH_SDA))
    assert res.max_segment_size == 4294967295
    assert res.device == 'sda'

    res_special = sys_block.MaxSegmentSize(context_wrap(MAX_SEGMENT_SIZE, SPECIAL_DEVICE_PATH))
    assert res_special.max_segment_size == 4294967295
    assert res_special.device == 'sd@!$'


def test_invalid_max_segment_size():
    with pytest.raises(ParseException) as e:
        sys_block.MaxSegmentSize(context_wrap(CONTENT_INVALID, MAX_SEGMENT_SIZE_PATH_SDA))
    assert "Error: " in str(e)

    with pytest.raises(ParseException) as e:
        sys_block.MaxSegmentSize(context_wrap(CONTENT_EMPTY))
    assert "Error: " in str(e)


def test_stable_writes_doc_examples():
    env = {
        'block_stable_writes': sys_block.StableWrites(context_wrap(STABLE_WRITES, FILE_PATH_SDA)),
        'max_segment_size': sys_block.MaxSegmentSize(context_wrap(MAX_SEGMENT_SIZE, MAX_SEGMENT_SIZE_PATH_SDA)),
    }
    failed, total = doctest.testmod(sys_block, globs=env)
    assert failed == 0



import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers.tomcat_virtual_dir_context import TomcatVirtualDirContextFallback, TomcatVirtualDirContextTargeted
from insights.tests import context_wrap

FOUND_1 = """
/usr/share/tomcat/conf/server.xml:    <Resources className="org.apache.naming.resources.VirtualDirContext"
""".strip()

FOUND_2 = """
/usr/share/tomcat/conf/server.xml:    <Resources className="org.apache.naming.resources.VirtualDirContext"
/usr/share/tomcat6/webapps/whatever/META-INF/context.xml:className="org.apache.naming.resources.VirtualDirContext"
""".strip()

FOUND_3 = """
/usr/share/tomcat/conf/server.xml:    <Resources className="org.apache.naming.resources.VirtualDirContext"
/usr/share/tomcat/conf/server.xml:"VirtualDirContext"
/usr/share/tomcat6/webapps/whatever/META-INF/context.xml:className="org.apache.naming.resources.VirtualDirContext"
""".strip()

NOT_FOUND = """
""".strip()

ERRORS_2 = """
garbage garbage
"""


def test_tomcat_virtual_dir_context_found():
    for parser in [TomcatVirtualDirContextFallback, TomcatVirtualDirContextTargeted]:
        tomcat_virtual_dir_context = parser(context_wrap(FOUND_1))
        assert len(tomcat_virtual_dir_context.data) == 1
        assert tomcat_virtual_dir_context.data == {'/usr/share/tomcat/conf/server.xml':
                                                   ['    <Resources className="org.apache.naming.resources.VirtualDirContext"'],
                                                   }

        tomcat_virtual_dir_context = parser(context_wrap(FOUND_2))
        assert len(tomcat_virtual_dir_context.data) == 2
        assert tomcat_virtual_dir_context.data == {'/usr/share/tomcat/conf/server.xml':
                                                   ['    <Resources className="org.apache.naming.resources.VirtualDirContext"'],
                                                   '/usr/share/tomcat6/webapps/whatever/META-INF/context.xml':
                                                   ['className="org.apache.naming.resources.VirtualDirContext"'],
                                                   }

        tomcat_virtual_dir_context = parser(context_wrap(FOUND_3))
        assert len(tomcat_virtual_dir_context.data) == 2
        assert tomcat_virtual_dir_context.data == {'/usr/share/tomcat/conf/server.xml':
                                                   ['    <Resources className="org.apache.naming.resources.VirtualDirContext"',
                                                    '"VirtualDirContext"'],
                                                   '/usr/share/tomcat6/webapps/whatever/META-INF/context.xml':
                                                   ['className="org.apache.naming.resources.VirtualDirContext"'],
                                                   }


def test_tomcat_virtual_dir_context_not_found():
    for parser in [TomcatVirtualDirContextFallback, TomcatVirtualDirContextTargeted]:
        with pytest.raises(SkipComponent) as excinfo:
            parser(context_wrap(NOT_FOUND))
            assert 'VirtualDirContext not used.' in str(excinfo.value)


def test_tomcat_virtual_dir_context_exceptions():
    for parser in [TomcatVirtualDirContextFallback, TomcatVirtualDirContextTargeted]:
        with pytest.raises(SkipComponent) as excinfo:
            parser(context_wrap(ERRORS_2))
            assert 'VirtualDirContext not used.' in str(excinfo.value)



import pkgutil

from collections import OrderedDict

from insights.core.exceptions import ParseException, SkipComponent  # noqa: F401


__all__ = [n for (i, n, p) in pkgutil.iter_modules(__path__) if not p]


def get_active_lines(lines, comment_char="#"):
    """
    Returns lines, or parts of lines, from content that are not commented out
    or completely empty.  The resulting lines are all individually stripped.

    This is useful for parsing many config files such as ifcfg.

    Parameters:
        lines (list): List of strings to parse.
        comment_char (str): String indicating that all chars following
            are part of a comment and will be removed from the output.

    Returns:
        list: List of valid lines remaining in the input.

    Examples:
        >>> lines = [
        ... 'First line',
        ... '   ',
        ... '# Comment line',
        ... 'Inline comment # comment',
        ... '          Whitespace          ',
        ... 'Last line']
        >>> get_active_lines(lines)
        ['First line', 'Inline comment', 'Whitespace', 'Last line']
    """
    return list(filter(None, (line.split(comment_char, 1)[0].strip() for line in lines)))


def optlist_to_dict(optlist, opt_sep=',', kv_sep='=', strip_quotes=False):
    """Parse an option list into a dictionary.

    Takes a list of options separated by ``opt_sep`` and places them into
    a dictionary with the default value of ``True``.  If ``kv_sep`` option
    is specified then key/value options ``key=value`` are parsed.  Useful
    for parsing options such as mount options in the format
    ``rw,ro,rsize=32168,xyz``.

    Parameters:
        optlist (str): String of options to parse.
        opt_sep (str): Separater used to split options.
        kv_sep (str): If not `None` then `optlist` includes key=value pairs
            to be split, and this str is used to split them.
        strip_quotes (bool): If set, will remove matching '"' and '"'
            characters from start and end of line.  No quotes are removed
            from inside the string and mismatched quotes are not removed.

    Returns:
        dict: Returns a dictionary of names present in the list.  If `kv_sep`
        is not `None` then the values will be the str on the right-hand side
        of `kv_sep`.  If `kv_sep` is `None` then each key will have a default
        value of `True`.

    Examples:
        >>> optlist = 'rw,ro,rsize=32168,xyz'
        >>> optlist_to_dict(optlist)
        {'rw': True, 'ro': True, 'rsize': '32168', 'xyz': True}
    """

    def make_kv(opt):
        if kv_sep is not None and kv_sep in opt:
            k, v = opt.split(kv_sep, 1)
            k = k.strip()
            if strip_quotes and v[0] in ('"', "'") and v[-1] == v[0]:
                return k, v[1:-1]
            else:
                return k, v
        else:
            return opt, True

    return dict(make_kv(opt) for opt in optlist.split(opt_sep))


def parse_cmdline_args(args_str, opt_sep=' '):
    """Parse a list of arguments into a dictionary.

    Takes a list of arguments separated by ``opt_sep`` and places them into
    a dictionary with the default value of ``True``.
    This is for parsing arguments such as command line optarguments, eg.
    ``ro rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff``.

    Parameters:
        args_str (str): String of arguments to parse.
        opt_sep (str): Separater used to split options, default to ' '

    Returns:
        dict: Returns a dictionary with arguments name as key, and a list of
            arguments value as value if argument be assigned with value, else,
            each key will have a default value of `True`.

    Examples:
        >>> args = 'ro rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff'
        >>> parse_cmdline_args(args)
        {'ro': [True], 'rd.lvm.lv': ['rhel/root', 'rhel/swap'], 'rhgb': [True],
        'quiet': [True], 'retbleed': ['stuff']}
    """
    parsed_args = {}
    for el in args_str.split(opt_sep):
        key, value = el, True
        if "=" in el:
            key, value = el.split("=", 1)
        if key not in parsed_args:
            parsed_args[key] = []
        parsed_args[key].append(value)
    return parsed_args


def split_kv_pairs(
    lines, comment_char="#", filter_string=None, split_on="=", use_partition=False, ordered=False
):
    """Split lines of a list into key/value pairs

    Use this function to filter and split all lines of a list of strings into
    a dictionary. Named arguments may be used to control how the line is split,
    how lines are filtered and the type of output returned.  See parameters for
    more information.  When splitting key/value, the first occurence of the
    split character is used, other occurrences of the split char in the line
    will be ignored. ::func:`get_active_lines` is called to strip comments and
    blank lines from the data.

    Parameters:
        lines (list of str): List of the strings to be split.
        comment_char (str): Char that when present in the line indicates all
            following chars are part of a comment.  If this is present, all
            comments and all blank lines are removed from list before further
            processing.  The default comment char is the `#` character.
        filter_string (str): If the filter string is present, then only lines
            containing the filter will be processed, other lines will be ignored.
        split_on (str): Character to use when splitting a line.  Only the first
            occurence of the char is used when splitting, so only one split is
            performed at the first occurrence of `split_on`.  The default string is `=`.
        use_partition (bool): If this parameter is `True` then the python `partition`
            function will be used to split the line. If `False` then the pyton `split`
            function will be used. The difference is that when `False`, if the split
            character is not present in the line then the line is ignored and when
            `True` the line will be parsed regardless. Set `use_partition` to `True`
            if you have valid lines that do not contain the `split_on` character.
            Set `use_partition` to `False` if you want to ignore lines that do not
            contain the `split_on` character. The default value is `False`.
        ordered (bool): If this parameter is `True` then the resulting dictionary
            will be in the same order as in the original file, a python
            `OrderedDict` type is used. If this parameter is `False` then the resulting
            dictionary is in no particular order, a base python `dict` type is used.
            The default is `False`.

    Returns:
        dict: Return value is a dictionary of the key/value pairs.  If parameter
        `keyword` is `True` then an OrderedDict is returned, otherwise a dict
        is returned.

    Examples:
        >>> from .. import split_kv_pairs
        >>> for line in lines:
        ...     print line
        # Comment line
        # Blank lines will also be removed
        keyword1 = value1   # Inline comments
        keyword2 = value2a=True, value2b=100M
        keyword3     # Key with no separator
        >>> split_kv_pairs(lines)
        {'keyword2': 'value2a=True, value2b=100M', 'keyword1': 'value1'}
        >>> split_kv_pairs(lines, comment_char='#')
        {'keyword2': 'value2a=True, value2b=100M', 'keyword1': 'value1'}
        >>> split_kv_pairs(lines, filter_string='keyword2')
        {'keyword2': 'value2a=True, value2b=100M'}
        >>> split_kv_pairs(lines, use_partition=True)
        {'keyword3': '', 'keyword2': 'value2a=True, value2b=100M', 'keyword1': 'value1'}
        >>> split_kv_pairs(lines, use_partition=True, ordered=True)
        OrderedDict([('keyword1', 'value1'), ('keyword2', 'value2a=True, value2b=100M'), ('keyword3', '')])

    """
    _lines = lines if comment_char is None else get_active_lines(lines, comment_char=comment_char)
    _lines = _lines if filter_string is None else [l for l in _lines if filter_string in l]
    kv_pairs = OrderedDict() if ordered else {}

    for line in _lines:
        if not use_partition:
            if split_on in line:
                k, v = line.split(split_on, 1)
                kv_pairs[k.strip()] = v.strip()
        else:
            k, _, v = line.partition(split_on)
            kv_pairs[k.strip()] = v.strip()
    return kv_pairs


def unsplit_lines(lines, cont_char='\\', keep_cont_char=False):
    r"""
    Recombine lines having a continuation character at end.

    Generator that recombines lines in the list that have the char `cont_char`
    at the end of a line.  If `cont_char` is found in a line then then
    next line will be appended to the current line, this will continue for
    multiple continuation lines until the next line is found with no
    continuation character at the end.  All lines found will be combined and
    returned.

    If the `keep_cont_char` option is set to True, the continuation character
    will be left on the end of the line.  Otherwise, by default, it is removed.

    Parameters:
        lines (list): List of strings to be evaluated.
        cont_char (char): Char to search for at end of line. Default is ``\\``.
        keep_cont_char (bool): Whether to keep the continuation on the end of
            the line.  Defaults to False, which causes the continuation
            character to be removed.

    Yields:
        line (str): Yields unsplit lines

    Examples:
        >>> lines = ['Line one \\', '     line one part 2', 'Line two']
        >>> list(unsplit_lines(lines))
        ['Line one      line one part 2', 'Line two']
        >>> list(unsplit_lines(lines, cont_char='2'))
        ['Line one \\', '     line one part Line two']
        >>> list(unsplit_lines(lines, keep_cont_char=True)
        ['Line one \     line one part 2', 'Line two']
    """
    unsplit_lines = []
    for line in lines:
        line = line.rstrip()
        if line.endswith(cont_char):
            unsplit_lines.append(line if keep_cont_char else line[:-1])
        else:
            yield ''.join(unsplit_lines) + line
            unsplit_lines = []
    if unsplit_lines:
        yield ''.join(unsplit_lines)


def calc_offset(lines, target, invert_search=False, require_all=False):
    """
    Function to search for a line in a list starting with a target string.
    If `target` is `None` or an empty string then `0` is returned.  This
    allows checking `target` here instead of having to check for an empty
    target in the calling function. Each line is stripped of leading spaces
    prior to comparison with each target however target is not stripped.
    See `parse_fixed_table` in this module for sample usage.

    Arguments:
        lines (list): List of strings.
        target (list): List of strings to search for at the beginning of any
            line in lines.
        invert_search (boolean): If `True` this flag causes the search to continue
            until the first line is found not matching anything in target.
            An empty line is implicitly included in target.  Default is `False`.
            This would typically be used if trimming trailing lines off of a
            file by passing `reversed(lines)` as the `lines` argument.
        require_all (boolean): If `True` this flag causes the search to *also*
            require all the items of the `target` being in the line.
            This flag only works with `invert_search == False`, when
            `invert_search` is `True`, it will be ignored.

    Returns:
        int: index into the `lines` indicating the location of `target`. If
        `target` is `None` or an empty string `0` is returned as the offset.
        If `invert_search` is `True` the index returned will point to the line
        after the last target was found.

    Raises:
        ValueError: Exception is raised if `target` string is specified and it
            was not found in the input lines.

    Examples:
        >>> lines = [
        ... '#   ',
        ... 'Warning line',
        ... 'Error line',
        ... '    data 1 line',
        ... '    data 2 line']
        >>> target = ['data', '2', 'line']
        >>> calc_offset(lines, target)
        3
        >>> target = ['#', 'Warning', 'Error']
        >>> calc_offset(lines, target, invert_search=True)
        3
        >>> target = ['data', '2', 'line']
        >>> calc_offset(lines, target, require_all=True)
        4
        >>> target = ['#', 'Warning', 'Error']
        >>> calc_offset(lines, target, invert_search=True, require_all=True)  # `require_all` doesn't work when `invert_search=True`
        3
    """
    if target and target[0] is not None:
        target = [t.strip() for t in target]
        for offset, line in enumerate(l.strip() for l in lines):
            # strip `target` string along with `line` value
            found_any = any([line.startswith(t) for t in target])
            if not invert_search and found_any:
                if require_all:
                    if all(t in line for t in target):
                        return offset
                else:
                    return offset
            elif invert_search and not (line == '' or found_any):
                return offset

        # If we get here then we didn't find any of the targets
        raise ValueError("Line containing '{}' was not found in table".format(','.join(target)))
    else:
        # If no target then return index 0
        return 0


def parse_fixed_table(
    table_lines, heading_ignore=[], header_substitute=[], trailing_ignore=[], empty_exception=False
):
    """
    Function to parse table data containing column headings in the first row and
    data in fixed positions in each remaining row of table data.
    Table columns must not contain spaces within the column name.  Column headings
    are assumed to be left justified and the column data width is the width of the
    heading label plus all whitespace to the right of the label. This function will
    remove all blank rows in data but it will handle blank columns if some of the
    columns aren't empty.

    Arguments:
        table_lines (list): List of strings with the first line containing column
            headings separated by spaces, and the remaining lines containing
            table data in left justified format.
        heading_ignore (list): Optional list of strings to search for at
            beginning of line.  All lines before this line will be ignored.
            If specified then it must be present in the file or `ValueError` will
            be raised.
        header_substitute (list): Optional list of tuples containing
            `(old_string_value, new_string_value)` to be used to modify header
            values.  If whitespace is present in a column it must be replaced with
            non-whitespace characters in order for the table to be parsed correctly.
        trailing_ignore (list): Optional list of strings to look for at the end
            rows of the content.  Lines starting with these strings will be ignored,
            thereby truncating the rows of data.
        empty_exception (bool): If True, raise a ParseException when the value if empty.
            False by default.

    Returns:
        list: Returns a list of dict for each row of column data.  Dict keys
            are the column headings in the same case as input.

    Raises:
        ValueError: Raised if `heading_ignore` is specified and not found in `table_lines`.
        ParseException: Raised if there are empty values when `empty_exception` is True

    Sample input::

        Column1    Column2    Column3
        data1      data 2     data   3
        data4      data5      data6

    Examples:
        >>> table_data = parse_fixed_table(table_lines)
        >>> table_data
        [{'Column1': 'data1', 'Column2': 'data 2', 'Column3': 'data   3'},
         {'Column1': 'data4', 'Column2': 'data5', 'Column3': 'data6'}]
    """

    def calc_column_indices(line, headers):
        idx = []
        for h in headers:
            i = idx[-1] + 1 if idx else 0
            idx.append(line.index(h, i))
        return idx

    first_line = calc_offset(table_lines, heading_ignore)
    try:
        last_line = len(table_lines) - calc_offset(
            reversed(table_lines), trailing_ignore, invert_search=True
        )
    except ValueError:
        last_line = len(table_lines)

    header = table_lines[first_line]
    if header_substitute:
        for old_val, new_val in header_substitute:
            header = header.replace(old_val, new_val)
    col_headers = header.strip().split()
    col_index = calc_column_indices(header, col_headers) + [None]
    idx_pairs = [(c, col_index[i + 1]) for i, c in enumerate(col_index) if c is not None]

    table_data = []
    for line in table_lines[first_line + 1 : last_line]:
        if line.strip():
            col_data = {}
            for i, (s, e) in enumerate(idx_pairs):
                val = line[s:e].strip()
                if empty_exception and not val:
                    raise ParseException('Incorrect line: \'{0}\''.format(line))
                col_data[col_headers[i]] = val
            table_data.append(col_data)

    return table_data


def parse_delimited_table(
    table_lines,
    delim=None,
    max_splits=-1,
    strip=True,
    header_delim='same as delimiter',
    heading_ignore=None,
    header_substitute=None,
    trailing_ignore=None,
    raw_line_key=None,
):
    """
    Parses table-like text.  Uses the first (non-ignored) row as the list of
    column names, which cannot contain the delimiter.  Fields cannot contain
    the delimiter but can be blank if a printable delimiter is used.

    Arguments:
        table_lines (list): List of strings with the first line containing
            column headings separated by spaces, and the remaining lines
            containing table data.
        delim (str): String used in the content to separate fields.  If left
            as None (the default), white space is used as the field separator.
        max_splits (int): Maximum number of fields to create by splitting the
            line.  After this number of fields has been found, the rest of the
            line is left un-split and may contain the delimiter.  Lines may
            contain less than this number of fields.
        strip (bool): If set to `True`, fields and headings will be stripped
            of leading and trailing space.  If set to `False`, fields and
            headings will be left as is.  The delimiter is always removed, so
            strip need not be set if `delim` is set to None (but will not
            change output in that case).
        header_delim (str): When set, uses a different delimiter to the
            content for splitting the header into keywords.  Set to `None`,
            this will split on white space.  When left at the special value
            of `'same as delimiter'`, the content delimiter will be used to
            split the header line as well.
        heading_ignore (list): Optional list of strings to search for at
            beginning of line.  All lines before this line will be ignored.
            If specified then it must be present in the file or `ValueError`
            will be raised.
        header_substitute (list): Optional list of tuples containing
            `(old_string_value, new_string_value)` to be used to modify
            header values.  If whitespace is present in a column it must be
            replaced with non-whitespace characters in order for the table to
            be parsed correctly.
        trailing_ignore (list): Optional list of strings to look for at the
            end rows of the content.  Lines starting with these strings will
            be ignored, thereby truncating the rows of data.
        raw_line_key (str): Key under which to save the raw line. If None, line
            is not saved.
    Returns:
        list: Returns a list of dictionaries for each row of column data,
        keyed on the column headings in the same case as input.

    """
    if not table_lines:
        return []
    first_line = calc_offset(table_lines, heading_ignore)
    try:
        # Ignore everything before the heading in this search
        last_line = len(table_lines) - calc_offset(
            reversed(table_lines[first_line + 1 :]), trailing_ignore, invert_search=True
        )
    except ValueError:
        # We seem to have run out of content before we found something we
        # wanted - return an empty list.
        return []

    if header_delim == 'same as delimiter':
        header_delim = delim
    header = table_lines[first_line]
    if header_substitute:
        for old_val, new_val in header_substitute:
            header = header.replace(old_val, new_val)

    content = table_lines[first_line + 1 : last_line]
    headings = [c.strip() if strip else c for c in header.split(header_delim)]
    r = []
    for line in content:
        row = line.strip()
        if row:
            rowsplit = row.split(delim, max_splits)
            if strip:
                rowsplit = [i.strip() for i in rowsplit]
            o = dict(zip(headings, rowsplit))
            if raw_line_key:
                o[raw_line_key] = line
            r.append(o)
    return r


def keyword_search(rows, parent=None, row_keys_change=False, **kwargs):
    """
    Takes a list of dictionaries and finds all the dictionaries where the
    keys and values match those found in the keyword arguments.

    Keys in the row data have ' ' and '-' replaced with '_', so they can
    match the keyword argument parsing.  For example, the keyword argument
    'fix_up_path' will match a key named 'fix-up path'. (see warning below)

    In addition, several suffixes can be added to the key name to do partial
    matching of values:

    * '__contains' will test whether the data value contains the given
      value.
    * '__startswith' tests if the data value starts with the given value
    * '__endswith' tests if the data value ends with the given value
    * '__lower_value' compares the lower-case version of the data and given
      values.

    Arguments:
        rows (list): A list of dictionaries representing the data to be
            searched.
        row_keys_change (bool): If True, each row might have different keys.
            This would happen if your data didn't add fields when the value
            was empty, or if combining different sources of data.  Most of
            the time it's safe to assume that the first row contains all of
            the keys, so if row_keys_change is False only the first row's
            keys are used as search keywords.
        **kwargs (dict): keyword-value pairs corresponding to the fields that
            need to be found and their required values in the data rows.

    Returns:
        (list): The list of rows that match the search keywords.  If no
        keyword arguments are given, no rows are returned.

    Examples:
        >>> rows = [
        ...     {'domain': 'oracle', 'type': 'soft', 'item': 'nofile', 'value': 1024},
        ...     {'domain': 'oracle', 'type': 'hard', 'item': 'nofile', 'value': 65536},
        ...     {'domain': 'oracle', 'type': 'soft', 'item': 'stack', 'value': 10240},
        ...     {'domain': 'oracle', 'type': 'hard', 'item': 'stack', 'value': 3276},
        ...     {'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': -1}]
        ...
        >>> keyword_search(rows, domain='root')
        [{'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': -1}]
        >>> keyword_search(rows, item__contains='c')
        [{'domain': 'oracle', 'type': 'soft', 'item': 'stack', 'value': 10240},
         {'domain': 'oracle', 'type': 'hard', 'item': 'stack', 'value': 3276},
         {'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': -1}]
        >>> keyword_search(rows, domain__startswith='r')
        [{'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': -1}]



    Testing has shown that caching the keyword_search() function itself does
    not result in much speed-up, but caching the key transformation does.  The
    cache is stored as an attribute, either on the object storing the rows or
    on a 'parent' object that can take an attribute (if 'rows' is a list, that
    cannot have an attribute added to it).  (We used to store the transformed
    dictionary of rows, but storing just the key transformations is faster.)
    """
    if not kwargs:
        return []
    if not rows:
        return []

    # Allows us to transform the key and do lookups like __contains and
    # __startswith
    matchers = {
        'equals': lambda s, v: s == v,
        'contains': lambda s, v: s is not None and v in s,
        'startswith': lambda s, v: s is not None and s.startswith(v),
        'endswith': lambda s, v: s is not None and s.endswith(v),
        'lower_value': lambda s, v: None not in (s, v) and s.lower() == v.lower(),
    }

    txform_cache_attr = '_transform_cache'
    if parent is None and hasattr(rows, '__dict__'):
        parent = rows
    # Uncomment this 'if' to check that all the parsers are supplying an
    # object, somehow, that can store our transformed row cache.
    # if parent is None:
    #     print("Invoked with no parent arg on primitive container - use parent=self argument to cache row transform")
    # The actual txkeys cache gets used rarely - in a basic test with a real
    # archive it only got used eleven times.  But that's a saving...
    if parent is not None and hasattr(parent, txform_cache_attr):
        txkeys = getattr(parent, txform_cache_attr)
    else:
        # Store the translation from the search key to the key in the data.
        all_keys = set()
        # Most data has the same keys in every row; but if row keys can change
        # between rows then we need to scan every row.
        if row_keys_change:
            for row in rows:
                # I tested this with a few different data scenarios and there
                # is no improvement if you check for the superset beforehand.
                all_keys.update(row.keys())
        else:
            # If your parser or combiner passes a dict_values for rows then you
            # need to turn it into a list...
            all_keys = set(list(rows[0].keys()))

        # Now build the 'transformed' key - the search keywords we recognise -
        # out of the keys we found.
        txkeys = dict((key.replace(' ', '_').replace('-', '_'), key) for key in all_keys)
        if parent is not None:
            setattr(parent, txform_cache_attr, txkeys)

    # pre-compile the kwargs to find the matcher function and underlying key.
    # Store these in a list of tuples for fast iteration and unpacking
    search_terms = list()
    for search_keyword, value in kwargs.items():
        # Again, we've tested this code with a variety of inputs and this
        # seems to be the fastest way:
        if '__' not in search_keyword:
            data_key = search_keyword
            matcher = 'equals'
        else:
            data_key, _, matcher = search_keyword.partition('__')
            if matcher not in matchers:
                # put key back the way we found it, matcher fn unchanged
                data_key = search_keyword
                matcher = 'equals'
        # If the data key sought is not in the row data, then we can say for
        # sure that the search will never match.  In the case of netstat,
        # where there are two different sections being searched and they do
        # not share keys, supplying a key that's only in one section is not
        # a coding error.
        if data_key not in txkeys:
            return []
        search_terms.append((txkeys[data_key], matcher, matchers[matcher], value))

    def key_match(row, data_key, matcher, matcher_fn, value):
        if matcher == 'equals':
            return data_key in row and row[data_key] == value
        return data_key in row and matcher_fn(row[data_key], value)

    data = list()
    for row in rows:
        if all(key_match(row, *term) for term in search_terms):
            data.append(row)
    return data



import pytest

from mock.mock import patch

from insights.combiners.redhat_release import RedHatRelease
from insights.core.exceptions import SkipComponent
from insights.parsers.redhat_release import RedhatRelease
from insights.specs.datasources.corosync import corosync_cmapctl_cmds
from insights.tests import context_wrap

COROSYNC_CMD_RHEL7 = [
    "/usr/sbin/corosync-cmapctl",
    "/usr/sbin/corosync-cmapctl -d runtime.schedmiss.timestamp",
    "/usr/sbin/corosync-cmapctl -d runtime.schedmiss.delay",
]

COROSYNC_CMD_RHEL9 = [
    "/usr/sbin/corosync-cmapctl",
    "/usr/sbin/corosync-cmapctl -m stats",
    "/usr/sbin/corosync-cmapctl -C schedmiss",
]

RHEL6 = "Red Hat Enterprise Linux Server release 6.5 (Santiago)"
RHEL7 = "Red Hat Enterprise Linux Server release 7.0 (Maipo)"
RHEL9 = "Red Hat Enterprise Linux release 9.0 (Plow)"


@patch('insights.specs.datasources.corosync.os.path.exists')
def test_corosync_cmapctl_cmds(path_exists):
    path_exists.return_value = True
    rhel = RedHatRelease(None, RedhatRelease(context_wrap(RHEL6)))
    broker = {RedHatRelease: rhel}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)

    rhel = RedHatRelease(None, RedhatRelease(context_wrap(RHEL7)))
    broker = {RedHatRelease: rhel}
    result = corosync_cmapctl_cmds(broker)
    assert result == COROSYNC_CMD_RHEL7

    rhel = RedHatRelease(None, RedhatRelease(context_wrap(RHEL9)))
    broker = {RedHatRelease: rhel}
    result = corosync_cmapctl_cmds(broker)
    assert result == COROSYNC_CMD_RHEL9


@patch('insights.specs.datasources.corosync.os.path.exists')
def test_corosync_cmapctl_cmds_no_such_cmd(path_exists):
    path_exists.return_value = False
    rhel = RedHatRelease(None, RedhatRelease(context_wrap(RHEL7)))
    broker = {RedHatRelease: rhel}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)

    rhel = RedHatRelease(None, RedhatRelease(context_wrap(RHEL9)))
    broker = {RedHatRelease: rhel}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)



"""
Collect all the interesting data for analysis - Core version
"""

from __future__ import absolute_import

import logging

from insights import collect
from insights.client.archive import InsightsArchive
from insights.client.constants import InsightsConstants as constants
from insights.client.utilities import systemd_notify_init_thread

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)


class CoreCollector(object):
    """
    Collector for core collection
    """

    def __init__(self, config):
        self.config = config
        self.archive = InsightsArchive(config)

    def run_collection(self, rm_conf):
        '''
        Initialize core collection here and generate the
        output directory with collected data.
        '''
        # initialize systemd-notify thread
        systemd_notify_init_thread()

        self.archive.create_archive_dir()

        logger.debug('Beginning to run core collection ...')

        self.config.rhsm_facts_file = constants.rhsm_facts_file
        collect.collect(
            tmp_path=self.archive.tmp_dir,
            archive_name=self.archive.archive_name,
            rm_conf=rm_conf or {},
            client_config=self.config,
        )

        logger.debug('Core collection finished.')

        # About "metadata":
        #  The following metadata is now collected by core collection when
        #  available, and will be stored under the "./data" directory instead
        #  of the root of the archive.
        #
        #   ansible_host = client_metadata.ansible_host
        #   blacklist_report = client_metadata.blacklist_report
        #   blacklisted_specs = client_metadata.blacklisted_specs
        #   branch_info = client_metadata.branch_info
        #   display_name = client_metadata.display_name
        #   egg_release = client_metadata.egg_release
        #   tags = client_metadata.tags
        #   version_info = client_metadata.version_info
        #
        #  See `insights.specs.datasources.client_metadata`

    def done(self):
        """
        Do finalization stuff
        """
        if self.config.output_dir:
            return self.archive.archive_dir
        else:
            return self.archive.create_tar_file()



import os

_user_home = os.path.expanduser('~')
_app_name = 'insights-client'
_uid = os.getuid()
_user_cache = os.getenv('XDG_CACHE_HOME', default=os.path.join(_user_home, '.cache'))


def _log_dir():
    '''
    Get the insights-client log dir

    Default: /var/log/insights-client
    Non-root user: $XDG_CACHE_HOME/insights-client || $HOME/.cache/insights-client/log
    '''
    if _uid == 0:
        insights_log_dir = os.path.join(os.sep, 'var', 'log', _app_name)
    else:
        insights_log_dir = os.path.join(_user_cache, _app_name, 'log')
    return insights_log_dir


def _lib_dir():
    '''
    Get the insights-client egg cache dir

    Default: /var/lib/insights
    Non-root user: $XDG_CACHE_HOME/insights-client || $HOME/.cache/insights-client/lib
    '''
    if _uid == 0:
        insights_lib_dir = os.path.join(os.sep, 'var', 'lib', 'insights')
    else:
        insights_lib_dir = os.path.join(_user_cache, _app_name, 'lib')
    return insights_lib_dir


class InsightsConstants(object):
    app_name = _app_name
    auth_method = 'BASIC'
    package_path = os.path.dirname(
        os.path.dirname(os.path.abspath(__file__)))
    sleep_time = 180
    command_blacklist = ('rm', 'kill', 'reboot', 'shutdown')
    default_conf_dir = os.getenv('INSIGHTS_CONF_DIR', default='/etc/insights-client')
    default_conf_file = os.path.join(default_conf_dir, 'insights-client.conf')
    default_tags_file = os.path.join(default_conf_dir, 'tags.yaml')
    log_dir = _log_dir()
    simple_find_replace_dir = '/etc/redhat-access-insights'
    default_log_file = os.path.join(log_dir, app_name + '.log')
    default_payload_log = os.path.join(log_dir, app_name + '-payload.log')
    custom_network_log_level = 11
    default_sed_file = os.path.join(default_conf_dir, '.exp.sed')
    base_url = 'cert-api.access.redhat.com/r/insights/platform'
    legacy_base_url = 'cert-api.access.redhat.com/r/insights'
    unregistered_files = [os.path.join(default_conf_dir, '.unregistered'),
                          os.path.join(simple_find_replace_dir, '.unregistered')]
    registered_files = [os.path.join(default_conf_dir, '.registered'),
                        os.path.join(simple_find_replace_dir, '.registered')]
    lastupload_file = os.path.join(default_conf_dir, '.lastupload')
    pub_gpg_path = os.path.join(default_conf_dir, 'redhattools.pub.gpg')
    machine_id_file = os.path.join(default_conf_dir, 'machine-id')
    default_branch_info = {'remote_branch': -1, 'remote_leaf': -1}
    default_cmd_timeout = 120  # default command execution to two minutes, prevents long running commands that will hang
    default_egg_gpg_key = os.path.join(default_conf_dir, 'insights-core.gpg')
    core_etag_file = os.path.join(default_conf_dir, '.insights-core.etag')
    core_gpg_sig_etag_file = os.path.join(default_conf_dir, '.insights-core-gpg-sig.etag')
    last_upload_results_file = os.path.join(default_conf_dir, '.last-upload.results')
    insights_core_lib_dir = _lib_dir()
    insights_core_rpm = os.path.join(default_conf_dir, 'rpm.egg')
    insights_core_last_stable = os.path.join(insights_core_lib_dir, 'last_stable.egg')
    insights_core_last_stable_gpg_sig = os.path.join(insights_core_lib_dir, 'last_stable.egg.asc')
    insights_core_newest = os.path.join(insights_core_lib_dir, 'newest.egg')
    insights_core_gpg_sig_newest = os.path.join(insights_core_lib_dir, 'newest.egg.asc')
    module_router_path = "/module-update-router/v1/channel?module=insights-core"
    sig_kill_ok = 100
    sig_kill_bad = 101
    cached_branch_info = os.path.join(default_conf_dir, '.branch_info')
    pidfile = os.path.join(os.sep, 'var', 'run', 'insights-client.pid')
    insights_tmp_path = os.path.join(os.sep, 'var', 'tmp')
    cache_dir = os.path.join(os.sep, 'var', 'cache', 'insights-client')
    insights_tmp_prefix = 'insights-client'
    egg_release_file = os.path.join(os.sep, cache_dir, 'insights-client-egg-release')
    ppidfile = os.path.join(os.sep, 'run', 'insights-client.ppid')
    valid_compressors = ("gz", "xz", "bz2", "none")
    # RPM version in which core collection was released
    core_collect_rpm_version = '3.1.0'
    # RPM version in which logrotate was released
    rpm_version_before_logrotate = '3.2.0'
    rhsm_facts_dir = os.path.join(os.sep, 'etc', 'rhsm', 'facts')
    rhsm_facts_file = os.path.join(os.sep, 'etc', 'rhsm', 'facts', 'insights-client.facts')
    # In MB
    archive_filesize_max = 100



import pytest
from insights.parsers.audit_log import AuditLog
from insights.tests import context_wrap
from datetime import date

AUDIT_LOG_TEMPALTE = """
type=CRYPTO_KEY_USER msg=audit(1506046832.641:53584): pid=16865 uid=0 auid=0 ses=7247 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 msg='op=destroy kind=session fp=? direction=both spid=16865 suid=0 rport=59296 laddr=192.0.2.1 lport=22  exe="/usr/sbin/sshd" hostname=? addr=10.66.136.139 terminal=? res=success'
type=LOGIN msg=audit(1506047401.407:53591): pid=482 uid=0 subj=system_u:system_r:crond_t:s0-s0:c0.c1023 old-auid=4294967295 auid=993 old-ses=4294967295 ses=7389 res=1
%s
type=CRED_REFR msg=audit(1508476956.471:13339): pid=30909 uid=0 auid=0 ses=1559 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 msg='op=PAM:setcred grantors=pam_unix acct="root" exe="/usr/sbin/sshd" hostname=foo.example.com addr=192.0.2.2 terminal=ssh res=success'
""".strip()

AUDIT_LOG_NORMAL = """
type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

AUDIT_LOG_START_WITH_SOME_WIRED_STRINGS = """
START WITH SOME WIRED STRINGS type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

AUDIT_LOG_WITHOUT_TIMESTAMP_FIELD = """
type=AVC msg=audit: avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

AUDIT_LOG_WITH_INVALID_TIMESTAMP_FIELD = """
type=AVC msg=audit(1506487181.009): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

AUDIT_LOG_WITH_JUST_HEAD_PART = """
type=AVC msg=audit(1506487181.009:32794):
""".strip()

AUDIT_LOG_WITHOUT_JUST_TAIL_PART = """
type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for
""".strip()

DUMMY_AUDIT_LOG_1 = """
type=AVC
""".strip()

DUMMY_AUDIT_LOG_2 = """
type=AVC msg=whatever(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

DUMMY_AUDIT_LOG_3 = """
type=AVC msg=audit(1506abc181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket
""".strip()

LAST_LINE_OF_TEMPLATE = """type=CRED_REFR msg=audit(1508476956.471:13339): pid=30909 uid=0 auid=0 ses=1559 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 msg='op=PAM:setcred grantors=pam_unix acct="root" exe="/usr/sbin/sshd" hostname=foo.example.com addr=192.0.2.2 terminal=ssh res=success'"""


def test_audit_log():

    auditlog = AuditLog(context_wrap(AUDIT_LOG_TEMPALTE % AUDIT_LOG_NORMAL))

    info = auditlog.get(['type=', 'msg='])
    assert len(info) == 4
    info = auditlog.get('type=')
    assert len(info) == 4
    line0 = info[0]
    assert line0.get('type') == 'CRYPTO_KEY_USER'
    assert line0.get('msg') == 'op=destroy kind=session fp=? direction=both spid=16865 suid=0 rport=59296 laddr=192.0.2.1 lport=22  exe="/usr/sbin/sshd" hostname=? addr=10.66.136.139 terminal=? res=success'
    assert line0.get('unparsed') is None
    line2 = info[2]
    assert line2.get('unparsed') == 'avc:  denied  { create } for'
    assert line2.get('comm') == 'mongod'
    assert line2.get('raw_message') == 'type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket'

    logtime = date.fromtimestamp(1506047401.407)
    logs = list(auditlog.get_after(timestamp=logtime))
    assert logs[0] == {'timestamp': '1506487181.009', 'is_valid': True,
            'unparsed': 'avc:  denied  { create } for',
            'msg_ID': '32794', 'pid': '27960', 'raw_message': AUDIT_LOG_NORMAL,
            'comm': 'mongod', 'scontext': 'system_u:system_r:mongod_t:s0',
            'tclass': 'unix_dgram_socket', 'type': 'AVC',
            'tcontext': 'system_u:system_r:mongod_t:s0'}
    assert logs[1]['raw_message'] == LAST_LINE_OF_TEMPLATE

    logs = list(auditlog.get_after(timestamp=logtime, s='AVC'))
    assert len(logs) == 1
    logs = list(auditlog.get_after(timestamp=logtime, s=['type=', 'AVC']))
    assert len(logs) == 1
    logs = list(auditlog.get_after(timestamp=logtime, s='NOTEXIST'))
    assert len(logs) == 0
    with pytest.raises(TypeError):
        list(auditlog.get_after(timestamp=logtime, s=['type=', False, 'AVC']))
    with pytest.raises(TypeError):
        list(auditlog.get_after(timestamp=logtime, s=set(['type=', 'AVC'])))

    options = [AUDIT_LOG_START_WITH_SOME_WIRED_STRINGS,
               AUDIT_LOG_WITH_INVALID_TIMESTAMP_FIELD,
               AUDIT_LOG_WITH_INVALID_TIMESTAMP_FIELD,
               DUMMY_AUDIT_LOG_1, DUMMY_AUDIT_LOG_2]

    for option in options:
        auditlog = AuditLog(context_wrap(AUDIT_LOG_TEMPALTE % option))
        info = auditlog.get('type=')
        assert len(info) == 4
        assert info[2].get('is_valid') is False
        assert info[3].get('type') == 'CRED_REFR'

        logtime = date.fromtimestamp(1506047401.407)
        logs = list(auditlog.get_after(timestamp=logtime))
        assert logs[0]['raw_message'] == LAST_LINE_OF_TEMPLATE

    auditlog = AuditLog(context_wrap(AUDIT_LOG_TEMPALTE % AUDIT_LOG_WITH_JUST_HEAD_PART))
    info = auditlog.get("type=AVC")
    assert len(info) == 1
    line0 = info[0]
    assert line0.get('unparsed') is None
    assert line0 == {'timestamp': '1506487181.009', 'msg_ID': '32794',
                     'is_valid': True, 'type': 'AVC',
                     'raw_message': 'type=AVC msg=audit(1506487181.009:32794):'}

    auditlog = AuditLog(context_wrap(AUDIT_LOG_TEMPALTE % AUDIT_LOG_WITHOUT_JUST_TAIL_PART))
    info = auditlog.get('type=')
    assert len(info) == 4
    line2 = info[2]
    assert line2.get('is_valid') is True
    assert line2.get('type') == 'AVC'
    assert line2.get('unparsed') == 'avc:  denied  { create } for'
    assert line2.get('raw_message') == 'type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for'

    auditlog = AuditLog(context_wrap(AUDIT_LOG_TEMPALTE % DUMMY_AUDIT_LOG_3))
    info = auditlog.get('type=')
    assert len(info) == 4
    logtime = date.fromtimestamp(1506047401.407)
    logs = list(auditlog.get_after(timestamp=logtime))
    assert logs[0]['raw_message'] == LAST_LINE_OF_TEMPLATE



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import httpd_M
from insights.parsers.httpd_M import HttpdM
from insights.tests import context_wrap

HTTPD_M_RHEL6 = """
httpd.event: apr_sockaddr_info_get() failed for liuxc-rhel6-apache
httpd.event: Could not reliably determine the server's fully qualified domain name, using 127.0.0.1 for ServerName
Loaded Modules:
 core_module (static)
 mpm_event_module (static)
 http_module (static)
 so_module (static)
 auth_basic_module (shared)
 auth_digest_module (shared)
 authn_file_module (shared)
 authn_alias_module (shared)
 authn_anon_module (shared)
 authn_dbm_module (shared)
 authn_default_module (shared)
 authz_host_module (shared)
 authz_user_module (shared)
 authz_owner_module (shared)
 authz_groupfile_module (shared)
 authz_dbm_module (shared)
 authz_default_module (shared)
 ldap_module (shared)
 authnz_ldap_module (shared)
 include_module (shared)
 log_config_module (shared)
Syntax OK
""".strip()

HTTPD_M_RHEL7 = """
AH00558: httpd: Could not reliably determine the server's fully qualified domain name, using fe80::a9:89fd:1fc4:f8d. Set the 'ServerName' directive globally to suppress this message
Loaded Modules:
 so_module (static)
 http_module (static)
 access_compat_module (shared)
 actions_module (shared)
 alias_module (shared)
 allowmethods_module (shared)
 auth_basic_module (shared)
 auth_digest_module (shared)
 authn_anon_module (shared)
 authn_core_module (shared)
 authn_dbd_module (shared)
 authn_dbm_module (shared)
 authn_file_module (shared)
""".strip()


HTTPD_M_DOC = """
Loaded Modules:
 core_module (static)
 http_module (static)
 access_compat_module (shared)
 actions_module (shared)
 alias_module (shared)
Syntax OK
""".strip()


def test_httpd_M():
    result = HttpdM(context_wrap(HTTPD_M_RHEL6, path='/usr/test/httpd_-M', args='/usr/test/httpd'))
    assert result.httpd_command == '/usr/test/httpd'
    assert sorted(result.loaded_modules) == sorted(result.shared_modules + result.static_modules)
    assert 'core_module' in result
    assert result['core_module'] == 'static'

    result = HttpdM(context_wrap(HTTPD_M_RHEL7, path='/usr/tst/httpd_-M', args='/usr/tst/httpd'))
    assert result.httpd_command == '/usr/tst/httpd'
    assert sorted(result.loaded_modules) == sorted(result.shared_modules + result.static_modules)
    assert 'core_module' not in result


def test_httpd_M_exp():
    with pytest.raises(ParseException) as sc:
        HttpdM(context_wrap(""))
    assert "Input content is empty." in str(sc)

    with pytest.raises(ParseException) as sc:
        HttpdM(context_wrap("HTTPD_M_24"))
    assert "Input content is not empty but there is no useful parsed data." in str(sc)


def test_httpd_M_doc():
    env = {
            'HttpdM': HttpdM,
            'hm': HttpdM(context_wrap(HTTPD_M_DOC, path='/usr/sbin/httpd_-M', args='/usr/sbin/httpd'))
          }
    failed, total = doctest.testmod(httpd_M, globs=env)
    assert failed == 0



# -*- coding: utf-8 -*-
"""
fwupdagent - Commands
=====================

FwupdagentDevices - command ``/bin/fwupdagent get-devices``
FwupdagentSecurity - command ``/bin/fwupdagent security --force``

Both commands produce JSON data representing current state of the HW firmware.
"""

from .. import parser, CommandParser, JSONParser
from insights.specs import Specs


@parser(Specs.fw_devices)
class FwupdagentDevices(CommandParser, JSONParser):
    """
    Class ``FwupdagentDevices`` parses the output of the ``/bin/fwupdagent get-devices`` command.

    Attributes:
        data (dict): The parsed output of the command.

    Sample input data::

        {
          "Devices" : [
            {
              "Name" : "Thunderbolt host controller",
              "DeviceId" : "113494429de0754484e6b93d70e879a913311dd5",
              "Guid" : [
                "ea3c7ac8-d937-568a-aab5-e053bd6af9ce",
                "3d3f953a-3408-56fa-81f3-042d73c59c16",
                "e773c51e-a20c-5b29-9f09-6bb0e0ef7560",
                "f1793149-6c12-5a7f-95db-9fefd116748d"
              ],
              "Summary" : "Unmatched performance for high-speed I/O",
              "Plugin" : "thunderbolt",
              "Protocol" : "com.intel.thunderbolt",
              "Flags" : [
                "internal",
                "updatable",
                "require-ac",
                "supported",
                "registered",
                "dual-image"
              ],
              "Vendor" : "Lenovo",
              "VendorId" : "THUNDERBOLT:0x0109|TBT:0x0109",
              "VendorIds" : [
                "THUNDERBOLT:0x0109",
                "TBT:0x0109"
              ],
              "Version" : "20.00",
              "VersionFormat" : "pair",
              "Icons" : [
                "thunderbolt"
              ],
              "Created" : 1633947746,
              "Releases" : [
                {
                  "AppstreamId" : "com.lenovo.ThinkPadN2JTF.firmware",
                  "RemoteId" : "lvfs",
                  "Summary" : "Lenovo ThinkPad X390/ThinkPad T490s Thunderbolt Firmware",
                  "Description" : "<p>Lenovo ThinkPad X390/ThinkPad T490s Thunderbolt Firmware</p><p>Fix Thunderbolt SPI ROM Wear out issue.</p>",
                  "Version" : "20.00",
                  "Filename" : "fc93d9ff3f7ced7eb11d9bf7d8cd009ce408d7d9",
                  "Protocol" : "com.intel.thunderbolt",
                  "Checksum" : [
                    "915a6657f6972937bcb88868e5d8ce1f1ec9fb85"
                  ],
                  "License" : "LicenseRef-proprietary",
                  "Size" : 253952,
                  "Created" : 1583253000,
                  "Locations" : [
                    "https://fwupd.org/downloads/8eef957c95cb6f534448be1faa7bbfc8702d620f64b757d40ee5e0b6b7094c0e-Lenovo-ThinkPad-X390-SystemFirmware-01.cab"
                  ],
                  "Uri" : "https://fwupd.org/downloads/8eef957c95cb6f534448be1faa7bbfc8702d620f64b757d40ee5e0b6b7094c0e-Lenovo-ThinkPad-X390-SystemFirmware-01.cab",
                  "Homepage" : "http://www.lenovo.com",
                  "Vendor" : "Lenovo Ltd."
                }
              ]
            },
            {
              "Name" : "USB3.0 Hub",
              "DeviceId" : "54f0d9041b6c5438c7ff825f5139559c5ca1b222",
              "Guid" : [
                "9429e4c7-f053-51d7-9289-75c4ddb14a97",
                "26f33695-3a3e-5c08-badb-f6141390ebd9",
                "10eb3a15-c177-5810-af53-1963e9200e65",
                "60e0a85e-b245-5c84-ba3b-d5bdb540cd47",
                "d91a45a0-4435-59d6-bb6b-9499f4a793c2",
                "022d2f73-4826-546a-ba0f-62579ea848ea"
              ],
              "Summary" : "USB 3.x Hub",
              "Plugin" : "vli",
              "Protocol" : "com.vli.usbhub",
              "Flags" : [
                "updatable",
                "registered",
                "can-verify",
                "can-verify-image",
                "dual-image",
                "self-recovery",
                "add-counterpart-guids"
              ],
              "Vendor" : "VIA Labs, Inc.",
              "VendorId" : "USB:0x2109",
              "Version" : "3.114",
              "VersionFormat" : "bcd",
              "Icons" : [
                "audio-card"
              ],
              "InstallDuration" : 15,
              "Created" : 1633947743
            }
          ]
        }

    Examples:
        >>> type(devices)
        <class 'insights.parsers.fwupdagent.FwupdagentDevices'>
        >>> len(devices["Devices"])
        2
    """
    pass


@parser(Specs.fw_security)
class FwupdagentSecurity(CommandParser, JSONParser):
    """
    Class ``FwupdagentSecurity`` parses the output of any of the commands:
        - ``/usr/bin/fwupdmgr security --force --json``
        - ``/bin/fwupdagent security --force``

    Attributes:
        data (dict): The parsed output of the command.

    Sample input data::

        {
          "HostSecurityAttributes" : [
            {
              "AppstreamId" : "org.fwupd.hsi.Kernel.Tainted",
              "HsiResult" : "not-tainted",
              "Name" : "Linux kernel",
              "Uri" : "https://fwupd.github.io/hsi.html#org.fwupd.hsi.Kernel.Tainted",
              "Flags" : [
                "success",
                "runtime-issue"
              ]
            },
            {
              "AppstreamId" : "org.fwupd.hsi.EncryptedRam",
              "HsiLevel" : 4,
              "HsiResult" : "not-supported",
              "Name" : "Encrypted RAM",
              "Uri" : "https://fwupd.github.io/hsi.html#org.fwupd.hsi.EncryptedRam"
            }
          ]
        }

    Examples:
        >>> type(security)
        <class 'insights.parsers.fwupdagent.FwupdagentSecurity'>
        >>> len(security["HostSecurityAttributes"])
        2
    """
    pass



import doctest
import pytest

from insights.core.exceptions import ContentException, ParseException, SkipComponent
from insights.parsers import satellite_content_hosts_count
from insights.tests import context_wrap


SATELLITE_CONTENT_HOSTS_COUNT = '''
 count
-------
    13
(1 row)

'''

SATELLITE_CONTENT_HOSTS_COUNT_WRONG_1 = '''
-bash: psql: command not found
'''

SATELLITE_CONTENT_HOSTS_COUNT_WRONG_2 = '''
su: user postgres does not exist
'''

SATELLITE_CONTENT_HOSTS_COUNT_WRONG_3 = '''
psql: FATAL:  database "foreman" does not exist
'''

SATELLITE_CONTENT_HOSTS_COUNT_WRONG_4 = '''
 count
-------
    abc
(1 row)

'''


def test_HTL_doc_examples():
    clients = satellite_content_hosts_count.SatelliteContentHostsCount(context_wrap(SATELLITE_CONTENT_HOSTS_COUNT))
    globs = {
        'clients': clients
    }
    failed, tested = doctest.testmod(satellite_content_hosts_count, globs=globs)
    assert failed == 0


def test_wrong_output():
    with pytest.raises(ContentException):
        satellite_content_hosts_count.SatelliteContentHostsCount(context_wrap(SATELLITE_CONTENT_HOSTS_COUNT_WRONG_1))
    with pytest.raises(SkipComponent):
        satellite_content_hosts_count.SatelliteContentHostsCount(context_wrap(SATELLITE_CONTENT_HOSTS_COUNT_WRONG_2))
    with pytest.raises(SkipComponent):
        satellite_content_hosts_count.SatelliteContentHostsCount(context_wrap(SATELLITE_CONTENT_HOSTS_COUNT_WRONG_3))
    with pytest.raises(ParseException):
        satellite_content_hosts_count.SatelliteContentHostsCount(context_wrap(SATELLITE_CONTENT_HOSTS_COUNT_WRONG_4))



# -*- coding: UTF-8 -*-
try:
    from unittest.mock import patch
except Exception:
    from mock import patch

from insights.client.client import collect
from insights.client.config import InsightsConfig
from insights.client.utilities import determine_hostname


def patch_get_rm_conf():
    """
    Mocks InsightsUploadConf.get_rm_conf so it returns a fixed configuration.
    """

    def decorator(old_function):
        patcher = patch("insights.client.client.InsightsUploadConf.get_rm_conf")
        return patcher(old_function)

    return decorator


def patch_core_collector():
    """
    Replaces CoreCollector with a dummy mock.
    """

    def decorator(old_function):
        patcher = patch("insights.client.client.CoreCollector")
        return patcher(old_function)

    return decorator


@patch_core_collector()
@patch_get_rm_conf()
@patch('insights.client.client.logger')
def test_log_msgs_general(log, get_rm_conf, core_collector):
    config = InsightsConfig()
    collect(config)
    get_rm_conf.assert_called_once_with()
    log.info.assert_called_once_with(
        'Starting to collect Insights data for %s' % determine_hostname(config.display_name)
    )


@patch_core_collector()
@patch_get_rm_conf()
@patch('insights.client.client.logger')
def test_log_msgs_compliance(log, get_rm_conf, core_collector):
    config = InsightsConfig(compliance=True)
    collect(config)
    get_rm_conf.assert_called_once_with()
    log.info.assert_called_once_with(
        'Starting to collect Insights data for %s' % determine_hostname(config.display_name)
    )


@patch_core_collector()
@patch_get_rm_conf()
@patch('insights.client.client.logger')
def test_log_msgs_compliance_policies(log, get_rm_conf, core_collector):
    config = InsightsConfig(compliance_policies=True)
    collect(config)
    get_rm_conf.assert_not_called()
    log.info.assert_not_called()


@patch_core_collector()
@patch_get_rm_conf()
@patch('insights.client.client.logger')
def test_log_msgs_compliance_assign(log, get_rm_conf, core_collector):
    config = InsightsConfig(compliance_assign=['abc'])
    collect(config)
    get_rm_conf.assert_not_called()
    log.info.assert_not_called()


@patch_core_collector()
@patch_get_rm_conf()
@patch('insights.client.client.logger')
def test_log_msgs_compliance_unassign(log, get_rm_conf, core_collector):
    config = InsightsConfig(compliance_unassign=['abc'])
    collect(config)
    get_rm_conf.assert_not_called()
    log.info.assert_not_called()



import doctest

import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import nvidia
from insights.parsers.nvidia import NvidiaSmiActiveClocksEventReasons, NvidiaSmiL, NvidiaSmiQueryGPU
from insights.tests import context_wrap

# ### test on NvidiaSmiL

NVIDIA_SMI_L_INPUT_1 = """
GPU 0: NVIDIA A100-PCIE-40GB (UUID: GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
GPU 1: NVIDIA A100-PCIE-40GB (UUID: GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a)
""".strip()


NVIDIA_SMI_L_INPUT_2 = """
GPU 0: NVIDIA T1000 (UUID: GPU-c05fe28c-5935-1c6d-3633-2fc61d26b6d4)
GPU 1: Tesla V100-PCIE-16GB (UUID: GPU-b08ecee0-0ea5-7b07-d459-baa5b95f5e89)
"""

NVIDIA_SMI_L_INPUT_3 = """
GPU 0: Quadro RTX 8000 (UUID: GPU-6b201039-6477-dd9c-1edd-137c8c44f8d6)
GPU 1: Quadro RTX 8000 (UUID: GPU-dbd81806-9040-9cc3-9eb9-87684fa2b7a4)
GPU 2: Quadro RTX 8000 (UUID: GPU-b4524d9e-858d-1050-1b4d-4fe0c94e8ea8)
GPU 3: Quadro RTX 8000 (UUID: GPU-51c72de5-fdc2-94b1-59e6-a12d4e961695)
GPU 4: Quadro RTX 8000 (UUID: GPU-bf443bbf-8598-8e63-e10c-e23ca737da84)
GPU 5: Quadro RTX 8000 (UUID: GPU-b20d26a5-5f6f-5c96-9d2e-ba6bfbb5d9a0)
GPU 6: Quadro RTX 8000 (UUID: GPU-5a497672-18a9-95c7-5b61-74353dd9c9cb)
GPU 7: Quadro RTX 8000 (UUID: GPU-9f690c37-bb84-1e1a-1e4a-3dbfd1b1015c)
""".strip()


def test_nvidia_smi_l():
    gpus = NvidiaSmiL(context_wrap(NVIDIA_SMI_L_INPUT_1))
    assert len(gpus) == 2
    assert gpus[0] == {
        "model": "NVIDIA A100-PCIE-40GB",
        "uuid": "GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838",
    }
    assert gpus.gpu_count == 2
    assert len(gpus.gpu_models) == 1
    assert "NVIDIA A100-PCIE-40GB" in gpus.gpu_models

    gpus = NvidiaSmiL(context_wrap(NVIDIA_SMI_L_INPUT_2))
    assert len(gpus) == 2
    assert gpus[0] == {"model": "NVIDIA T1000", "uuid": "GPU-c05fe28c-5935-1c6d-3633-2fc61d26b6d4"}
    assert gpus.gpu_count == 2
    assert len(gpus.gpu_models) == 2
    assert "Tesla V100-PCIE-16GB" in gpus.gpu_models

    gpus = NvidiaSmiL(context_wrap(NVIDIA_SMI_L_INPUT_3))
    assert len(gpus) == 8
    assert gpus[0] == {
        "model": "Quadro RTX 8000",
        "uuid": "GPU-6b201039-6477-dd9c-1edd-137c8c44f8d6",
    }
    assert gpus.gpu_count == 8


NVIDIA_SMI_L_UNPARSABLE_0 = """
some other extra line  # will be skipped in parsing
""".strip()

NVIDIA_SMI_L_UNPARSABLE_1 = """
some other extra line  # will be skipped in parsing
GPU 0: NVIDIA A100-PCIE-40GB ( GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
GPU 1 NVIDIA A100-PCIE-40GB (UUID: GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a)
""".strip()

NVIDIA_SMI_L_UNPARSABLE_2 = """
GPU 0: NVIDIA A100-PCIE-40GB (UUID: GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
GPU 2: NVIDIA A100-PCIE-40GB (UUID: GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a)
""".strip()

NVIDIA_SMI_L_UNPARSABLE_3 = """
GPU 0:  (UUID: GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
GPU 1: NVIDIA A100-PCIE-40GB (UUID: GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a)
""".strip()

NVIDIA_SMI_L_UNPARSABLE_4 = """
GPU 0: NVIDIA A100-PCIE-40GB (UUID: GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
GPU 1: NVIDIA A100-PCIE-40GB (UUID: )
""".strip()


def test_nvidia_smi_l_exceptions():
    with pytest.raises(SkipComponent) as err:
        NvidiaSmiL(context_wrap(" "))
    assert "Empty content" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiL(context_wrap(NVIDIA_SMI_L_UNPARSABLE_0))
    assert "Empty GPU info after parse" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiL(context_wrap(NVIDIA_SMI_L_UNPARSABLE_1))
    assert "Unparsable GPU model:" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiL(context_wrap(NVIDIA_SMI_L_UNPARSABLE_2))
    assert "Unparsable GPU id:" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiL(context_wrap(NVIDIA_SMI_L_UNPARSABLE_3))
    assert "Unparsable GPU line:" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiL(context_wrap(NVIDIA_SMI_L_UNPARSABLE_4))
    assert "Unparsable GPU line:" in str(err)


# ### test on NvidiaSmiActiveClocksEventReasons

ACTIVE_CLOCKS_EVENT_REASONS = """
NVIDIA L4, 0x0000000000000001
NVIDIA A1, 0x0000000000000000
NVIDIA H1, 0x0000000000000084
""".strip()


def test_nvidia_smi_active_clocks_event_reasons():
    active_clocks_event_reasons = NvidiaSmiActiveClocksEventReasons(
        context_wrap(ACTIVE_CLOCKS_EVENT_REASONS)
    )
    assert len(active_clocks_event_reasons) == 3
    assert active_clocks_event_reasons[0]['gpu_name'] == "NVIDIA L4"
    assert not active_clocks_event_reasons[0]['none']
    assert active_clocks_event_reasons[1]['none']
    assert not active_clocks_event_reasons[2]['none']
    assert active_clocks_event_reasons[2]['sw_power_cap']
    assert active_clocks_event_reasons[2]['hw_power_brake_slowdown']
    assert not active_clocks_event_reasons[2]['hw_slowdown']


INVALID_ACTIVE_CLOCKS_EVENT_REASONS_1 = """
NVIDIA L4, 0000000000000001
""".strip()

INVALID_ACTIVE_CLOCKS_EVENT_REASONS_2 = """
NVIDIA L4
""".strip()

INVALID_ACTIVE_CLOCKS_EVENT_REASONS_3 = """
NVIDIA L4, 0x0000000000000001

NVIDIA A1, 0x0000000000000000
""".strip()

INVALID_ACTIVE_CLOCKS_EVENT_REASONS_4 = """
NVIDIA L4, 0x0000000000000001
NVIDIA H1, 0000000000000084
NVIDIA A1, 0x0000000000000000
""".strip()

INVALID_ACTIVE_CLOCKS_EVENT_REASONS_5 = """


"""


def test_nvidia_smi_active_clocks_event_reasons_exceptions():
    with pytest.raises(ParseException) as err:
        NvidiaSmiActiveClocksEventReasons(context_wrap(INVALID_ACTIVE_CLOCKS_EVENT_REASONS_1))
    assert "Not an expected command output for active clocks event reasons" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiActiveClocksEventReasons(context_wrap(INVALID_ACTIVE_CLOCKS_EVENT_REASONS_2))
    assert "Not an expected command output for active clocks event reasons" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiActiveClocksEventReasons(context_wrap(INVALID_ACTIVE_CLOCKS_EVENT_REASONS_3))
    assert "Not an expected command output for active clocks event reasons" in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiActiveClocksEventReasons(context_wrap(INVALID_ACTIVE_CLOCKS_EVENT_REASONS_4))
    assert "Not an expected command output for active clocks event reasons" in str(err)

    with pytest.raises(SkipComponent) as err:
        NvidiaSmiActiveClocksEventReasons(context_wrap(INVALID_ACTIVE_CLOCKS_EVENT_REASONS_5))
    assert "Empty content." in str(err)


# ### test on NvidiaSmiQueryGPU

NVIDIA_SMI_QUERY_GPU_DATA_1 = """
0, NVIDIA L4, GPU-24598a07-f97d-fc79-86de-485c4c82d01c, 23034 MiB
1, NVIDIA A1, GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838, 16280 MiB
2, NVIDIA H1, GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a, 24564 MiB
"""
NVIDIA_SMI_QUERY_GPU_DATA_EX_EXTRA_SPACE = """
0,   NVIDIA L4  , GPU-24598a07-f97d-fc79-86de-485c4c82d01c, 23034 MiB
1, NVIDIA A1, GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838, 16280 MiB
"""

NVIDIA_SMI_QUERY_GPU_DATA_EX_1 = """
0 NVIDIA L4, GPU-24598a07-f97d-fc79-86de-485c4c82d01c, 23034 MiB
"""
NVIDIA_SMI_QUERY_GPU_DATA_EX_MISSING_FIELD = """
0, NVIDIA L4, GPU-24598a07-f97d-fc79-86de-485c4c82d01c
1, NVIDIA A1, GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838, 16280 MiB
"""
NVIDIA_SMI_QUERY_GPU_DATA_EX_EMPTY_FIELD = """
0, NVIDIA L4, , 23034 MiB
1, NVIDIA A1, GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838, 16280 MiB
"""


def test_nvidia_smi_query_gpu():
    gpus = NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_1))
    assert len(gpus) == 3
    assert gpus.gpu_count == 3
    assert gpus[0].index == '0'
    assert gpus[0].model == 'NVIDIA L4'
    assert gpus[0].uuid == 'GPU-24598a07-f97d-fc79-86de-485c4c82d01c'
    assert gpus[0].memory_total == '23034 MiB'
    assert gpus[2].model == 'NVIDIA H1'

    gpus = NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_EX_EXTRA_SPACE))
    assert len(gpus) == 2


def test_nvidia_smi_query_gpu_exceptions():
    with pytest.raises(SkipComponent) as err:
        NvidiaSmiQueryGPU(context_wrap(""))

    with pytest.raises(ParseException) as err:
        NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_EX_1))
    assert "Not an expected command output: " in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_EX_MISSING_FIELD))
    assert "Not an expected command output: " in str(err)

    with pytest.raises(ParseException) as err:
        NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_EX_EMPTY_FIELD))
    assert "Not an expected command output: " in str(err)


def test_nvidia_doc_examples():
    env = {
        'gpus': NvidiaSmiL(context_wrap(NVIDIA_SMI_L_INPUT_1)),
        'active_clocks_event_reasons': NvidiaSmiActiveClocksEventReasons(
            context_wrap(ACTIVE_CLOCKS_EVENT_REASONS)
        ),
        'gpus_info': NvidiaSmiQueryGPU(context_wrap(NVIDIA_SMI_QUERY_GPU_DATA_1)),
    }
    failed, total = doctest.testmod(nvidia, globs=env)
    assert failed == 0



"""
Clean Specs (files/commands)
============================

The following modules are provided in the Cleaner and can be applied to the
specs during collection according to the user configuration and specs setting.

- Redaction (patterns redaction)
  This is a must-be-done operation to all the collected specs.  A `no_redact`
  option is available to specs, if it's surely contains non-security
  information, e.g. the `machine-id` spec.

- Filtering
  Filter lines as per the allow list got from the `filters.yaml`.  The
  `filtering` can only be applied when `allowlist` is available (not None) for
  the spec.

- Obfuscation (IPv4, [IPv6], Hostname, MAC, Password, Keywords)
  Obfuscate lines in spec content according to the user configuration and
  specs requirement.  The `no_obfuscate` can be used to exclude obfuscation
  target from the obfuscation.  Currently, the supported obfuscation target
  are:
  * hostname
  * ipv4
  * ipv6
  * keyword
  * mac
  * password
"""

import logging
import json
import os
import six
import tempfile

from insights.cleaner.filters import AllowFilter
from insights.cleaner.hostname import Hostname
from insights.cleaner.ip import IPv4, IPv6
from insights.cleaner.keyword import Keyword

from insights.cleaner.mac import Mac
from insights.cleaner.password import Password
from insights.cleaner.pattern import Pattern
from insights.cleaner.utilities import write_report
from insights.util.hostname import determine_hostname
from insights.util.posix_regex import replace_posix

logger = logging.getLogger(__name__)
MAX_LINE_LENGTH = 1048576  # 1MB
DEFAULT_OBFUSCATIONS = {
    'hostname',
    'ipv4',
    'ipv6',
    'keyword',
    'mac',
    'password',
}


class Cleaner(object):
    """
    Class to clean the content of Specs according to the user configuration and
    spec setting.
    """

    def __init__(self, config, rm_conf, fqdn=None):
        self.report_dir = tempfile.gettempdir()
        self.rhsm_facts_file = getattr(
            config, 'rhsm_facts_file', os.path.join(self.report_dir, 'insights-client.facts')
        )
        # Handle User Configuration
        rm_conf = rm_conf or {}
        exclude = rm_conf.get('patterns', [])
        regex = False
        if isinstance(exclude, dict) and exclude.get('regex'):
            exclude = [r'%s' % replace_posix(i) for i in exclude['regex']]
            regex = True
        # - Pattern Redaction and allow-list Filtering
        self.redact = {
            'pattern': Pattern(exclude, regex) if exclude else None,
            'allow_filter': AllowFilter(),
        }
        # - Keyword and Password Replacement
        #   They Do NOT depend on "obfuscation_list"
        keywords = rm_conf.get('keywords')
        self.obfuscate = {
            'keyword': Keyword(keywords) if keywords else None,
            'password': Password(),
        }

        self.fqdn = fqdn if fqdn else determine_hostname()
        obfs = config.obfuscation_list if config else None
        if config and obfs:
            # - IPv4 obfuscation
            self.obfuscate.update(ipv4=IPv4()) if 'ipv4' in obfs else None
            # - IPv6 obfuscation
            self.obfuscate.update(ipv6=IPv6()) if 'ipv6' in obfs else None
            # - Hostname obfuscation
            self.obfuscate.update(hostname=Hostname(self.fqdn)) if 'hostname' in obfs else None
            # - MAC obfuscation
            self.obfuscate.update(mac=Mac()) if 'mac' in obfs else None

    def clean_content(self, lines, no_obfuscate=None, no_redact=False, allowlist=None, width=False):
        """
        Clean lines one by one according to the configuration.

        For some extra large files, e.g. logs, we want to keep the bottom
        part of them.  So the lines are processed in reverse order.  But the
        processed result is returned in the original order.
        """

        def _clean_line(line):
            if len(line) > MAX_LINE_LENGTH:
                # Keep the first MAX_LINE_LENGTH chars only (it rarely happens)
                line = line[:MAX_LINE_LENGTH]
                logger.debug('Extra-long line is truncated ...')

            for parser, kwargs in parsers:
                line = parser.parse_line(line, **kwargs)
            return line

        # List of parsers to be applied with Order
        parsers = list()
        # 1. Redact when NO "no_redact=True" is set
        if self.redact['pattern'] and not no_redact:
            parsers.append((self.redact['pattern'], {})) if not no_redact else None
        # 2. Filter as per allowlist got from add_filter  # copy it to avoid write back
        (
            parsers.append((self.redact['allow_filter'], {'allowlist': dict(allowlist)}))
            if allowlist is not None
            else None
        )
        # 3. Obfuscation entries
        # - Hostname
        # - IPv4
        # - IPv6
        # - Keyword
        # - Mac
        # - Password
        for obf in set(self.obfuscate.keys()) - set(no_obfuscate or []):
            if self.obfuscate[obf]:
                parsers.append((self.obfuscate[obf], {'width': width}))

        # handle single string
        if not isinstance(lines, list):
            return _clean_line(lines)

        result = []
        # process lines in reverse order
        for idx in range(len(lines) - 1, -1, -1):
            line = _clean_line(lines[idx])
            result.append(line) if line is not None else None
        if result and any(l for l in result):
            # When some lines Truthy, return them in right order
            result.reverse()
            return result
        # All lines blank
        return []

    def clean_file(self, _file, no_obfuscate=None, no_redact=False, allowlist=None):
        """
        Clean a file according to the configuration, the file will be updated
        directly with the cleaned content.
        """
        logger.debug('Cleaning %s ...' % _file)

        if os.path.exists(_file) and not os.path.islink(_file):
            # Process the file
            raw_data = content = None
            try:
                with open(_file, 'r') as fh:
                    raw_data = fh.readlines()
                    content = self.clean_content(
                        raw_data,
                        no_obfuscate=no_obfuscate,
                        no_redact=no_redact,
                        allowlist=allowlist,
                        width=_file.endswith("netstat_-neopa"),
                    )
            except Exception as e:  # pragma: no cover
                logger.warning(e)
                raise Exception("Error: Cannot Open File for Cleaning: %s" % _file)
            # Store it
            try:
                if raw_data:
                    if content:
                        with open(_file, 'wb') as fh:
                            for line in content:
                                fh.write(line.encode('utf-8') if six.PY3 else line)
                    else:
                        # Remove Empty file
                        logger.debug('Removing %s, as it\'s empty after cleaning' % _file)
                        os.remove(_file)
            except Exception as e:  # pragma: no cover
                logger.warning(e)
                raise Exception("Error: Cannot Write to File: %s" % _file)

    def generate_rhsm_facts(self):
        logger.info('Writing RHSM facts to %s ...', self.rhsm_facts_file)

        hostname = self.obfuscate.get('hostname')
        hn_mapping = hostname.mapping() if hostname else []

        keyword = self.obfuscate.get('keyword')
        kw_mapping = keyword.mapping() if keyword else []

        ipv4 = self.obfuscate.get('ipv4')
        ipv4_mapping = ipv4.mapping() if ipv4 else []

        ipv6 = self.obfuscate.get('ipv6')
        ipv6_mapping = ipv6.mapping() if ipv6 else []

        mac = self.obfuscate.get('mac')
        mac_mapping = mac.mapping() if mac else []

        facts = {
            'insights_client.hostname': self.fqdn,
            'insights_client.obfuscate_ipv4_enabled': 'ipv4' in self.obfuscate,
            'insights_client.obfuscate_ipv6_enabled': 'ipv6' in self.obfuscate,
            'insights_client.obfuscate_hostname_enabled': 'hostname' in self.obfuscate,
            'insights_client.obfuscate_mac_enabled': 'mac' in self.obfuscate,
            'insights_client.obfuscated_ipv4': json.dumps(ipv4_mapping),
            'insights_client.obfuscated_ipv6': json.dumps(ipv6_mapping),
            'insights_client.obfuscated_mac': json.dumps(mac_mapping),
            'insights_client.obfuscated_hostname': json.dumps(hn_mapping),
            'insights_client.obfuscated_keyword': json.dumps(kw_mapping),
        }

        write_report(facts, self.rhsm_facts_file)

    def generate_report(self, archive_name):
        # Always generate the rhsm.facts files
        self.generate_rhsm_facts()
        # Generate CSV reports accordingly
        for parser in list(self.redact.values()) + list(self.obfuscate.values()):
            if parser:
                parser.generate_report(self.report_dir, archive_name)



try:
    from unittest.mock import patch
except Exception:
    from mock import patch


from insights.specs.datasources.logrotate import logrotate_conf_list


@patch("os.listdir", return_value=[])
def test_logrotate_conf_list_empty(listdir):
    ret = logrotate_conf_list({})
    assert ret == ["/etc/logrotate.conf"]


@patch("os.path.isdir", return_value=False)
def test_logrotate_conf_list_no_such_d(isdir):
    ret = logrotate_conf_list({})
    assert ret == ["/etc/logrotate.conf"]


@patch("os.listdir", return_value=['a.conf', 'b', 'c.cc', 'd.o.conf', 'e.o.c', '.f.conf', '.g'])
@patch("os.path.isfile", return_value=True)
def test_logrotate_conf_list(isfile, listdir):
    ret = logrotate_conf_list({})
    assert len(ret) == 5
    assert "/etc/logrotate.conf" in ret
    assert "/etc/logrotate.d/a.conf" in ret
    assert "/etc/logrotate.d/b" in ret
    assert '/etc/logrotate.d/d.o.conf' in ret
    assert '/etc/logrotate.d/.f.conf' in ret



"""
DotNet- Comand ``/usr/bin/dotnet``
==================================

The parsers related ``/usr/bin/dotnet --version`` is included in this module.

DotNetVersion - command ``dotnet --version``
--------------------------------------------

ContainerDotNetVersion - command ``dotnet --version`` for containers
--------------------------------------------------------------------
"""
from insights.core import CommandParser, ContainerParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.dotnet_version)
class DotNetVersion(CommandParser):
    """
    Class for parsing the output of the ``/usr/bin/dotnet --version`` command.

    Sample output::

        3.1.108

    Examples:
        >>> dotnet_ver.major
        3
        >>> dotnet_ver.minor
        1
        >>> dotnet_ver.raw
        '3.1.108'
    """

    def parse_content(self, content):
        if not content or len(content) > 1:
            raise SkipComponent

        self.major = self.minor = None
        self.raw = content[0].strip()

        if ' ' not in self.raw and '.' in self.raw:
            v_sp = [i.strip() for i in self.raw.split('.', 2)]
            if len(v_sp) >= 2 and v_sp[0].isdigit() and v_sp[1].isdigit():
                self.major = int(v_sp[0])
                self.minor = int(v_sp[1])

        if self.major is None:
            raise ParseException("Unrecognized version: {0}", self.raw)


@parser(Specs.container_dotnet_version)
class ContainerDotNetVersion(ContainerParser, DotNetVersion):
    """
    Parses the output of the ``/usr/bin/dotnet --version`` command of the running
    containers which are based on RHEL images.

    Sample output::

        3.1.108

    Examples:
        >>> type(con_dotnet_ver)
        <class 'insights.parsers.dotnet.ContainerDotNetVersion'>
        >>> con_dotnet_ver.container_id
        'cc2883a1a369'
        >>> con_dotnet_ver.image
        'quay.io/rhel8'
        >>> con_dotnet_ver.engine
        'podman'
        >>> con_dotnet_ver.major
        3
        >>> con_dotnet_ver.minor
        1
        >>> con_dotnet_ver.raw
        '3.1.108'
    """

    pass
